@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: help.c,v 1.95 2003/05/15 19:42:31 n8gray Exp $";
+static const char CVSID[] = "$Id: help.c,v 1.102 2004/08/01 10:06:10 yooden Exp $";
 /*******************************************************************************
 *                                                                              *
 * help.c -- Nirvana Editor help display                                        *
@@ -8,7 +8,8 @@ static const char CVSID[] = "$Id: help.c,v 1.95 2003/05/15 19:42:31 n8gray Exp $
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.                                                                     *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
 *                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
@@ -174,7 +175,7 @@ static unsigned char AlphabetToAsciiTable[256];
 /*============================================================================*/
 
 static Widget createHelpPanel(enum HelpTopic topic);
-static void dismissCB(Widget w, XtPointer clientData, XtPointer callData);
+static void closeCB(Widget w, XtPointer clientData, XtPointer callData);
 static void prevTopicCB(Widget w, XtPointer clientData, XtPointer callData);
 static void nextTopicCB(Widget w, XtPointer clientData, XtPointer callData);
 static void bwHistoryCB(Widget w, XtPointer clientData, XtPointer callData);
@@ -268,16 +269,17 @@ static const char *getBuildInfo(void)
         locale = setlocale(LC_MESSAGES, "");
 
         sprintf(bldInfoString, bldFormat,
-             NEditVersion,
-             COMPILE_OS, COMPILE_MACHINE, COMPILE_COMPILER,
-             linkdate, linktime,
-             XmVERSION, XmREVISION, XmUPDATE_LEVEL,
-             XmVERSION_STRING,
-             xmUseVersion/1000, xmUseVersion%1000,
-             _XmVersionString,
-             ServerVendor(TheDisplay), VendorRelease(TheDisplay),
-             visualStr,
-             locale ? locale : "None");
+            NEditVersion,
+            COMPILE_OS, COMPILE_MACHINE, COMPILE_COMPILER,
+            linkdate, linktime,
+            XmVERSION, XmREVISION, XmUPDATE_LEVEL,
+            XmVERSION_STRING,
+            xmUseVersion/1000, xmUseVersion%1000,
+            _XmVersionString,
+            (NULL == TheDisplay ? "<unknown>" : ServerVendor(TheDisplay)),
+            (NULL == TheDisplay ? 0 : VendorRelease(TheDisplay)),
+            visualStr,
+            locale ? locale : "None");
 
         atexit(freeBuildInfo);
     }
@@ -574,7 +576,7 @@ static Widget createHelpPanel(enum HelpTopic topic)
 {
     Arg al[50];
     int ac;
-    Widget appShell, btn, dismissBtn, form, btnFW;
+    Widget appShell, btn, closeBtn, form, btnFW;
     Widget sw, hScrollBar, vScrollBar;
     XmString st1;
     char * helpText  = NULL;
@@ -582,8 +584,8 @@ static Widget createHelpPanel(enum HelpTopic topic)
 
     ac = 0;
     XtSetArg(al[ac], XmNdeleteResponse, XmDO_NOTHING); ac++;
-    appShell = CreateShellWithBestVis(APP_NAME, APP_CLASS,
-            applicationShellWidgetClass, TheDisplay, al, ac);
+    appShell = CreateWidget(TheAppShell, "help",
+            topLevelShellWidgetClass, al, ac);
     AddSmallIcon(appShell);
     /* With openmotif 2.1.30, a crash may occur when the text widget of the
        help window is (slowly) resized to a zero width. By imposing a 
@@ -632,16 +634,16 @@ static Widget createHelpPanel(enum HelpTopic topic)
     XtAddCallback(btn, XmNactivateCallback, printCB, appShell);
     XmStringFree(st1);
 
-    dismissBtn = XtVaCreateManagedWidget("dismiss",
+    closeBtn = XtVaCreateManagedWidget("close",
             xmPushButtonWidgetClass, form,
-            XmNlabelString, st1=XmStringCreateSimple("Dismiss"),
+            XmNlabelString, st1=XmStringCreateSimple("Close"),
             XmNbottomAttachment, XmATTACH_FORM,
             XmNleftAttachment, XmATTACH_POSITION,
             XmNleftPosition, 75,
             XmNrightAttachment, XmATTACH_POSITION,
             XmNrightPosition, 97,
             NULL);
-    XtAddCallback(dismissBtn, XmNactivateCallback, dismissCB, appShell);
+    XtAddCallback(closeBtn, XmNactivateCallback, closeCB, appShell);
     XmStringFree(st1);
             
     /* Create the next row of buttons (for navigation) */
@@ -649,7 +651,7 @@ static Widget createHelpPanel(enum HelpTopic topic)
             XmNlabelString, st1=XmStringCreateSimple("<< Browse"),
             XmNmnemonic, 'o', 
             XmNbottomAttachment, XmATTACH_WIDGET,
-            XmNbottomWidget, dismissBtn,
+            XmNbottomWidget, closeBtn,
             XmNleftAttachment, XmATTACH_POSITION,
             XmNleftPosition, 51,
             XmNrightAttachment, XmATTACH_POSITION,
@@ -662,7 +664,7 @@ static Widget createHelpPanel(enum HelpTopic topic)
             XmNlabelString, st1=XmStringCreateSimple("Browse >>"),
             XmNmnemonic, 'e', 
             XmNbottomAttachment, XmATTACH_WIDGET,
-            XmNbottomWidget, dismissBtn,
+            XmNbottomWidget, closeBtn,
             XmNleftAttachment, XmATTACH_POSITION,
             XmNleftPosition, 75, 
             XmNrightAttachment, XmATTACH_POSITION,
@@ -675,7 +677,7 @@ static Widget createHelpPanel(enum HelpTopic topic)
             XmNlabelString, st1=XmStringCreateSimple("Back"),
             XmNmnemonic, 'B', 
             XmNbottomAttachment, XmATTACH_WIDGET,
-            XmNbottomWidget, dismissBtn,
+            XmNbottomWidget, closeBtn,
             XmNleftAttachment, XmATTACH_POSITION,
             XmNleftPosition, 3,
             XmNrightAttachment, XmATTACH_POSITION,
@@ -688,7 +690,7 @@ static Widget createHelpPanel(enum HelpTopic topic)
             XmNlabelString, st1=XmStringCreateSimple("Forward"),
             XmNmnemonic, 'w', 
             XmNbottomAttachment, XmATTACH_WIDGET,
-            XmNbottomWidget, dismissBtn,
+            XmNbottomWidget, closeBtn,
             XmNleftAttachment, XmATTACH_POSITION,
             XmNleftPosition, 27,
             XmNrightAttachment, XmATTACH_POSITION,
@@ -716,8 +718,12 @@ static Widget createHelpPanel(enum HelpTopic topic)
             XmNorientation, XmVERTICAL,
             XmNrepeatDelay, 10,
             NULL);
+    /* Make sure the fixed size help font is loaded, such that we can base
+       our text widget size calculation on it. */
+    loadFontsAndColors(sw, 'A');
     HelpTextPanes[topic] = XtVaCreateManagedWidget("helpText",
             textWidgetClass, sw,
+            textNfont, HelpStyleInfo[0].font, /* MUST correspond to 'A' above */
             textNrows, 30,
             textNcolumns, 65,
             textNbacklightCharTypes, NULL,
@@ -758,8 +764,8 @@ static Widget createHelpPanel(enum HelpTopic topic)
     AddDialogMnemonicHandler(form, FALSE);
     
     /* Set the default button */
-    XtVaSetValues(form, XmNdefaultButton, dismissBtn, NULL);
-    XtVaSetValues(form, XmNcancelButton, dismissBtn, NULL);
+    XtVaSetValues(form, XmNdefaultButton, closeBtn, NULL);
+    XtVaSetValues(form, XmNcancelButton, closeBtn, NULL);
     
     /* realize all of the widgets in the new window */
     RealizeWithoutForcingPosition(appShell);
@@ -768,7 +774,7 @@ static Widget createHelpPanel(enum HelpTopic topic)
     XmProcessTraversal(HelpTextPanes[topic], XmTRAVERSE_CURRENT);
 
     /* Make close command in window menu gracefully prompt for close */
-    AddMotifCloseCallback(appShell, (XtCallbackProc)dismissCB, appShell);
+    AddMotifCloseCallback(appShell, (XtCallbackProc)closeCB, appShell);
     
     /* Initialize navigation information, if it hasn't already been init'd */
     initNavigationHistory();
@@ -802,7 +808,7 @@ static void changeTopicOrRaise(int existingTopic, int newTopic) {
 /*
 ** Callbacks for window controls
 */
-static void dismissCB(Widget w, XtPointer clientData, XtPointer callData)
+static void closeCB(Widget w, XtPointer clientData, XtPointer callData)
 {
     int topic;
     
@@ -956,7 +962,7 @@ static int is_known_link(char *link_name, int *topic, int *textPosition)
 ** Find the text of a hyperlink from a clicked character position somewhere
 ** within the hyperlink text, and display the help that it links to.
 */
-static void follow_hyperlink(int topic, int charPosition, int newWindow)
+static void followHyperlink(int topic, int charPosition, int newWindow)
 {
     textDisp *textD = ((TextWidget)HelpTextPanes[topic])->text.textD;
     char * link_text;
@@ -1130,7 +1136,7 @@ static void helpHyperlinkAP(Widget w, XEvent *event, String *args,
     if (*nArgs == 3)
         XtCallActionProc(w, args[1], event, NULL, 0);
     
-    follow_hyperlink(topic, clickedPos, newWin);
+    followHyperlink(topic, clickedPos, newWin);
 }  
  
 /*
@@ -1185,7 +1191,7 @@ static void searchHelpText(Widget parent, int parentTopic,
             searchHelpText(parent, parentTopic, searchFor, allSections, 0, 0);
             return;
         }
-        DialogF(DF_INF, parent, 1, "String Not Found", "String Not Found", "Dismiss");
+        DialogF(DF_INF, parent, 1, "String Not Found", "String Not Found", "OK");
         return;
     }
     
