@@ -1,10 +1,59 @@
-/* $Id: nedit.h,v 1.17 2001/05/31 08:49:52 amai Exp $ */
+/* $Id: nedit.h,v 1.58 2004/08/20 19:33:20 n8gray Exp $ */
 /*******************************************************************************
-*									       *
-* nedit.h -- Nirvana Editor common include file				       *
-*									       *
+*                                                                              *
+* nedit.h -- Nirvana Editor Common Header File                                 *
+*                                                                              *
+* Copyright 2004 The NEdit Developers                                          *
+*                                                                              *
+* This is free software; you can redistribute it and/or modify it under the    *
+* terms of the GNU General Public License as published by the Free Software    *
+* Foundation; either version 2 of the License, or (at your option) any later   *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+*                                                                              *
+* This software is distributed in the hope that it will be useful, but WITHOUT *
+* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* for more details.                                                            *
+*                                                                              *
+* You should have received a copy of the GNU General Public License along with *
+* software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
+* Place, Suite 330, Boston, MA  02111-1307 USA                                 *
+*                                                                              *
+* Nirvana Text Editor                                                          *
+* July 31, 2001                                                                *
+*                                                                              *
 *******************************************************************************/
 
+#ifndef NEDIT_NEDIT_H_INCLUDED
+#define NEDIT_NEDIT_H_INCLUDED
+
+#include "textBuf.h"
+
+#include <X11/Intrinsic.h>
+#include <Xm/Xm.h>
+#include <Xm/XmStrDefs.h>
+#ifdef VMS
+#include "../util/VMSparam.h"
+#else
+#ifndef __MVS__
+#include <sys/param.h>
+#endif
+#endif /*VMS*/
+
+/* Some default colors */
+#define NEDIT_DEFAULT_FG        "black"
+#define NEDIT_DEFAULT_TEXT_BG   "rgb:e5/e5/e5"
+#define NEDIT_DEFAULT_SEL_FG    "black"
+#define NEDIT_DEFAULT_SEL_BG    "rgb:cc/cc/cc"
+#define NEDIT_DEFAULT_HI_FG     "white" /* These are colors for flashing */
+#define NEDIT_DEFAULT_HI_BG     "red"   /*   matching parens. */
+#define NEDIT_DEFAULT_LINENO_FG "black"
+#define NEDIT_DEFAULT_CURSOR_FG "black"
+#define NEDIT_DEFAULT_HELP_FG   "black"
+#define NEDIT_DEFAULT_HELP_BG   "rgb:cc/cc/cc"
+
+
 /* Tuning parameters */
 #define SEARCHMAX 511		/* Maximum length of search/replace strings */
 #define MAX_SEARCH_HISTORY 100	/* Maximum length of search string history */
@@ -19,14 +68,11 @@
 #define AUTOSAVE_OP_LIMIT 8	/* number of distinct editing operations user
 				   can do before NEdit gens. new backup file */
 #define MAX_FONT_LEN 100	/* maximum length for a font name */
+#define MAX_COLOR_LEN 30	/* maximum length for a color name */
 #define MAX_MARKS 36	    	/* max. # of bookmarks (one per letter & #) */
 #define MIN_LINE_NUM_COLS 4 	/* Min. # of columns in line number display */
 #define APP_NAME "nedit"	/* application name for loading resources */
 #define APP_CLASS "NEdit"	/* application class for loading resources */
-#ifdef SGI_CUSTOM
-#define SGI_WINDOW_TITLE "nedit: " /* part of window title string for sgi */
-#define SGI_WINDOW_TITLE_LEN 7     /* length of SGI_WINDOW_TITLE */
-#endif
 
 /* The accumulated list of undo operations can potentially consume huge
    amounts of memory.  These tuning parameters determine how much undo infor-
@@ -47,13 +93,14 @@
 #define UNDO_OP_TRIMTO 200	  /* size undo list is normally trimmed to
 				     when it exceeds UNDO_OP_TRIMTO in length */
 #ifdef SGI_CUSTOM
-#define MAX_SHORTENED_ITEMS 40    /* max. number of items excluded in short- */
+#define MAX_SHORTENED_ITEMS 100   /* max. number of items excluded in short- */
 #endif	    	    	    	  /*     menus mode */
 
 enum indentStyle {NO_AUTO_INDENT, AUTO_INDENT, SMART_INDENT};
 enum wrapStyle {NO_WRAP, NEWLINE_WRAP, CONTINUOUS_WRAP};
-enum fileFormats {UNIX_FILE_FORMAT, DOS_FILE_FORMAT, MAC_FILE_FORMAT};
 enum showMatchingStyle {NO_FLASH, FLASH_DELIMIT, FLASH_RANGE};
+enum virtKeyOverride { VIRT_KEY_OVERRIDE_NEVER, VIRT_KEY_OVERRIDE_AUTO,
+                       VIRT_KEY_OVERRIDE_ALWAYS };
 
 #define NO_FLASH_STRING		"off"
 #define FLASH_DELIMIT_STRING	"delimiter"
@@ -81,8 +128,7 @@ enum showMatchingStyle {NO_FLASH, FLASH_DELIMIT, FLASH_RANGE};
 /* This handles all the different reasons files can be locked */
 #define USER_LOCKED_BIT     0
 #define PERM_LOCKED_BIT     1
-#define FORCE_LOCKED_BIT    2
-#define TOO_MUCH_BINARY_DATA_LOCKED_BIT 3
+#define TOO_MUCH_BINARY_DATA_LOCKED_BIT 2
 
 #define LOCKED_BIT_TO_MASK(bitNum) (1 << (bitNum))
 #define SET_LOCKED_BY_REASON(reasons, onOrOff, reasonBit) ((onOrOff) ? \
@@ -93,8 +139,6 @@ enum showMatchingStyle {NO_FLASH, FLASH_DELIMIT, FLASH_RANGE};
 #define SET_USER_LOCKED(reasons, onOrOff) SET_LOCKED_BY_REASON(reasons, onOrOff, USER_LOCKED_BIT)
 #define IS_PERM_LOCKED(reasons) (((reasons) & LOCKED_BIT_TO_MASK(PERM_LOCKED_BIT)) != 0)
 #define SET_PERM_LOCKED(reasons, onOrOff) SET_LOCKED_BY_REASON(reasons, onOrOff, PERM_LOCKED_BIT)
-#define IS_FORCE_LOCKED(reasons) (((reasons) & LOCKED_BIT_TO_MASK(FORCE_LOCKED_BIT)) != 0)
-#define SET_FORCE_LOCKED(reasons, onOrOff) SET_LOCKED_BY_REASON(reasons, onOrOff, FORCE_LOCKED_BIT)
 #define IS_TMBD_LOCKED(reasons) (((reasons) & LOCKED_BIT_TO_MASK(TOO_MUCH_BINARY_DATA_LOCKED_BIT)) != 0)
 #define SET_TMBD_LOCKED(reasons, onOrOff) SET_LOCKED_BY_REASON(reasons, onOrOff, TOO_MUCH_BINARY_DATA_LOCKED_BIT)
 
@@ -103,6 +147,9 @@ enum showMatchingStyle {NO_FLASH, FLASH_DELIMIT, FLASH_RANGE};
 #define IS_ANY_LOCKED(reasons) ((reasons) != 0)
 #define CLEAR_ALL_LOCKS(reasons) ((reasons) = 0)
 
+/* determine a safe size for a string to hold an integer-like number contained in xType */
+#define TYPE_INT_STR_SIZE(xType) ((sizeof(xType) * 3) + 2)
+
 /* Record on undo list */
 typedef struct _UndoInfo {
     struct _UndoInfo *next;		/* pointer to the next undo record */
@@ -128,20 +175,116 @@ typedef struct {
     selection sel;
 } Bookmark;
 
+/* Identifiers for the different colors that can be adjusted. */
+enum colorTypes {
+    TEXT_FG_COLOR,
+    TEXT_BG_COLOR,
+    SELECT_FG_COLOR,
+    SELECT_BG_COLOR,
+    HILITE_FG_COLOR,
+    HILITE_BG_COLOR,
+    LINENO_FG_COLOR,
+    CURSOR_FG_COLOR,
+    NUM_COLORS
+};
+
+/* cache user menus: manage mode of user menu list element */
+typedef enum {
+    UMMM_UNMANAGE,     /* user menu item is unmanaged */
+    UMMM_UNMANAGE_ALL, /* user menu item is a sub menu and is
+                          completely unmanaged (including nested
+                          sub menus) */
+    UMMM_MANAGE,       /* user menu item is managed; menu items
+                          of potential sub menu are (un)managed
+                          individually */
+    UMMM_MANAGE_ALL    /* user menu item is a sub menu and is
+                          completely managed */
+} UserMenuManageMode;
+
+/* structure representing one user menu item */
+typedef struct _UserMenuListElement {
+    UserMenuManageMode    umleManageMode;          /* current manage mode */
+    UserMenuManageMode    umlePrevManageMode;      /* previous manage mode */
+    char                 *umleAccKeys;             /* accelerator keys of item */
+    Boolean               umleAccLockPatchApplied; /* indicates, if accelerator
+                                                      lock patch is applied */
+    Widget                umleMenuItem;            /* menu item represented by
+                                                      this element */
+    Widget                umleSubMenuPane;         /* holds menu pane, if item
+                                                      represents a sub menu */
+    struct _UserMenuList *umleSubMenuList;         /* elements of sub menu, if
+                                                      item represents a sub menu */
+} UserMenuListElement;
+
+/* structure holding a list of user menu items */
+typedef struct _UserMenuList {
+    int                   umlNbrItems;
+    UserMenuListElement **umlItems;
+} UserMenuList;
+
+/* structure holding cache info about Shell and Macro menus, which are
+   shared over all "tabbed" documents (needed to manage/unmanage this
+   user definable menus when language mode changes) */
+typedef struct _UserMenuCache {
+    int          umcLanguageMode;     /* language mode applied for shared
+                                         user menus */
+    Boolean      umcShellMenuCreated; /* indicating, if all shell menu items
+                                         were created */
+    Boolean      umcMacroMenuCreated; /* indicating, if all macro menu items
+                                         were created */
+    UserMenuList umcShellMenuList;    /* list of all shell menu items */
+    UserMenuList umcMacroMenuList;    /* list of all macro menu items */
+} UserMenuCache;
+
+/* structure holding cache info about Background menu, which is
+   owned by each document individually (needed to manage/unmanage this
+   user definable menu when language mode changes) */
+typedef struct _UserBGMenuCache {
+    int          ubmcLanguageMode;    /* language mode applied for background
+                                         user menu */
+    Boolean      ubmcMenuCreated;     /* indicating, if all background menu
+                                         items were created */
+    UserMenuList ubmcMenuList;        /* list of all background menu items */
+} UserBGMenuCache;
+
+/* The WindowInfo structure holds the information on a Document. A number
+   of 'tabbed' documents may reside within a shell window, hence some of 
+   its members are of 'shell-level'; namely the find/replace dialogs, the
+   menu bar & its associated members, the components on the stats area
+   (i-search line, statsline and tab bar), plus probably a few others.
+   See CreateWindow() and CreateDocument() for more info.
+   
+   Each document actually 'lives' within its splitPane widget member, 
+   which can be raised to become the 'top' (visible) document by function
+   RaiseDocument(). The non-top documents may still be accessed through
+   macros, or the context menu on the tab bar.
+   
+   Prior to the introduction of tabbed mode, each window may house only 
+   one document, making it effectively an 'editor window', hence the name
+   WindowInfo. This struct name has been preserved to ease the transition
+   when tabbed mode was introduced after NEdit 5.4.
+*/
 typedef struct _WindowInfo {
     struct _WindowInfo *next;
     Widget	shell;			/* application shell of window */
+    Widget	mainWin;		/* main window of shell */
     Widget	splitPane;		/* paned win. for splitting text area */
     Widget	textArea;		/* the first text editing area widget */
     Widget	textPanes[MAX_PANES];	/* additional ones created on demand */
     Widget	lastFocus;		/* the last pane to have kbd. focus */
     Widget	statsLine;		/* file stats information display */
+    Widget      statsLineForm;
+    Widget      statsLineColNo;         /* Line/Column information display */
     Widget  	iSearchForm;	    	/* incremental search line widgets */
+    Widget  	iSearchFindButton;
     Widget  	iSearchText;
+    Widget  	iSearchClearButton;
     Widget  	iSearchRegexToggle;
     Widget  	iSearchCaseToggle;
     Widget  	iSearchRevToggle;
     Widget	menuBar;    	    	/* the main menu bar */
+    Widget	tabBar;			/* tab bar for tabbed window */
+    Widget	tab;			/* tab for this document */
     Widget	replaceDlog;		/* replace dialog */
     Widget	replaceText;		/* replace dialog settable widgets... */
     Widget	replaceWithText;
@@ -174,39 +317,56 @@ typedef struct _WindowInfo {
     Widget	replaceMultiFileList;
     Widget	replaceMultiFilePathBtn;
     Widget	fontDialog;		/* NULL, unless font dialog is up */
+    Widget	colorDialog;		/* NULL, unless color dialog is up */
     Widget	readOnlyItem;		/* menu bar settable widgets... */
     Widget	autoSaveItem;
     Widget	saveLastItem;
+    Widget      openSelItem;
+    Widget      newOppositeItem;
     Widget	closeItem;
     Widget	printSelItem;
     Widget	undoItem;
     Widget	redoItem;
     Widget	cutItem;
+    Widget	delItem;
     Widget	copyItem;
+    Widget	lowerItem;
+    Widget	upperItem;
+    Widget      findSelItem;
+    Widget      findAgainItem;
+    Widget	replaceFindAgainItem;
+    Widget 	replaceAgainItem;
+    Widget      gotoSelItem;
     Widget	langModeCascade;
     Widget	findDefItem;
+    Widget	showTipItem;
     Widget	autoIndentOffItem;
     Widget	autoIndentItem;
     Widget	smartIndentItem;
     Widget  	noWrapItem;
     Widget  	newlineWrapItem;
     Widget  	continuousWrapItem;
+    Widget	showTabBarItem;
     Widget	statsLineItem;
     Widget	iSearchLineItem;
     Widget	lineNumsItem;
     Widget	showMatchingOffItem;
     Widget	showMatchingDelimitItem;
     Widget	showMatchingRangeItem;
+    Widget	matchSyntaxBasedItem;
     Widget	overtypeModeItem;
     Widget	highlightItem;
     Widget	windowMenuPane;
     Widget	shellMenuPane;
     Widget	macroMenuPane;
     Widget  	bgMenuPane;
+    Widget  	tabMenuPane;
     Widget  	prevOpenMenuPane;
     Widget  	prevOpenMenuItem;
     Widget  	unloadTagsMenuPane;
     Widget  	unloadTagsMenuItem;
+    Widget  	unloadTipsMenuPane;
+    Widget  	unloadTipsMenuItem;
     Widget	filterItem;
     Widget	autoIndentOffDefItem;
     Widget	autoIndentDefItem;
@@ -219,21 +379,33 @@ typedef struct _WindowInfo {
     Widget	showMatchingOffDefItem;
     Widget	showMatchingDelimitDefItem;
     Widget	showMatchingRangeDefItem;
+    Widget	matchSyntaxBasedDefItem;
     Widget	highlightOffDefItem;
     Widget	highlightDefItem;
+    Widget	backlightCharsItem;
+    Widget	backlightCharsDefItem;
     Widget	searchDlogsDefItem;
     Widget      beepOnSearchWrapDefItem;
     Widget	keepSearchDlogsDefItem;
     Widget	searchWrapsDefItem;
+    Widget      appendLFItem;
     Widget	sortOpenPrevDefItem;
     Widget	allTagsDefItem;
     Widget	smartTagsDefItem;
     Widget	reposDlogsDefItem;
+    Widget      autoScrollDefItem;
+    Widget	openInTabDefItem;
+    Widget	tabBarDefItem;
+    Widget	tabBarHideDefItem;
+    Widget	toolTipsDefItem;
+    Widget	tabNavigateDefItem;
+    Widget      tabSortDefItem;
     Widget	statsLineDefItem;
     Widget	iSearchLineDefItem;
     Widget	lineNumsDefItem;
     Widget	pathInWindowsMenuDefItem;
     Widget  	modWarnDefItem;
+    Widget  	modWarnRealDefItem;
     Widget  	exitWarnDefItem;
     Widget	searchLiteralDefItem;
     Widget	searchCaseSenseDefItem;
@@ -257,8 +429,12 @@ typedef struct _WindowInfo {
     Widget	cancelMacroItem;
     Widget	replayItem;
     Widget	repeatItem;
-    Widget	splitWindowItem;
+    Widget	splitPaneItem;
     Widget	closePaneItem;
+    Widget	detachDocumentItem;
+    Widget	moveDocumentItem;
+    Widget	contextMoveDocumentItem;
+    Widget	contextDetachDocumentItem;
     Widget  	bgMenuUndoItem;
     Widget  	bgMenuRedoItem;
 #ifdef SGI_CUSTOM
@@ -270,7 +446,7 @@ typedef struct _WindowInfo {
 #endif
     char	filename[MAXPATHLEN];	/* name component of file being edited*/
     char	path[MAXPATHLEN];	/* path component of file being edited*/
-    int		fileMode;		/* permissions of file being edited */
+    unsigned	fileMode;		/* permissions of file being edited */
     int     	fileFormat; 	    	/* whether to save the file straight
     	    	    	    	    	   (Unix format), or convert it to
 					   MS DOS style with \r\n line breaks */
@@ -303,6 +479,7 @@ typedef struct _WindowInfo {
     					   of selection related menu items */
     Boolean	filenameSet;		/* is the window still "Untitled"? */ 
     Boolean	fileChanged;		/* has window been modified? */
+    Boolean     fileMissing;            /* is the window's file gone? */
     int         lockReasons;            /* all ways a file can be locked */
     Boolean	autoSave;		/* is autosave turned on? */
     Boolean	saveOldVersion;		/* keep old version in filename.bck */
@@ -313,15 +490,19 @@ typedef struct _WindowInfo {
     char 	showMatchingStyle; 	/* How to show matching parens:
 					   NO_FLASH, FLASH_DELIMIT, or
 					   FLASH_RANGE */
+    char	matchSyntaxBased;	/* Use syntax info to show matching */
     Boolean	showStats;		/* is stats line supposed to be shown */
     Boolean 	showISearchLine;    	/* is incr. search line to be shown */
     Boolean 	showLineNumbers;    	/* is the line number display shown */
-    Boolean 	showPathInWindowsMenu; 	/* is the path shown in windows menu */
     Boolean	highlightSyntax;	/* is syntax highlighting turned on? */
+    Boolean	backlightChars;		/* is char backlighting turned on? */
+    char	*backlightCharTypes;	/* what backlighting to use */
     Boolean	modeMessageDisplayed;	/* special stats line banner for learn
     					   and shell command executing modes */
+    char	*modeMessage;		/* stats line banner content for learn
+    					   and shell command executing modes */
     Boolean	ignoreModify;		/* ignore modifications to text area */
-    Boolean	windowMenuValid;	/* is window menu is up to date? */
+    Boolean	windowMenuValid;	/* is window menu up to date? */
     Boolean	prevOpenMenuValid;	/* Prev. Opened Files menu up to date?*/
     int		rHistIndex, fHistIndex;	/* history placeholders for */
     int     	iSearchHistIndex;	/*   find and replace dialogs */
@@ -335,6 +516,7 @@ typedef struct _WindowInfo {
     	    	    	    	    	   info. about it, otherwise, NULL */
     void    	*macroCmdData;  	/* same for macro commands */
     void    	*smartIndentData;   	/* compiled macros for smart indent */
+    Atom	fileClosedAtom;         /* Atom used to tell nc that the file is closed */
     int    	languageMode;	    	/* identifies language mode currently
     	    	    	    	    	   selected in the window */
     Boolean	multiFileReplSelected;	/* selected during last multi-window 
@@ -361,9 +543,16 @@ typedef struct _WindowInfo {
     Widget	replaceScopeSelToggle;	/* Scope for replace = selection */
     Widget	replaceScopeMultiToggle;/* Scope for replace = multiple files */
 #endif
+    UserMenuCache   *userMenuCache;     /* cache user menus: */
+    UserBGMenuCache  userBGMenuCache;   /* shell & macro menu are shared over all
+                                           "tabbed" documents, while each document
+                                           has its own background menu. */
 } WindowInfo;
 
 extern WindowInfo *WindowList;
 extern Display *TheDisplay;
+extern Widget TheAppShell;
 extern char *ArgV0;
 extern Boolean IsServer;
+
+#endif /* NEDIT_NEDIT_H_INCLUDED */
