@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: selection.c,v 1.25 2003/05/09 17:43:48 edg Exp $";
+static const char CVSID[] = "$Id: selection.c,v 1.31 2004/08/01 10:06:11 yooden Exp $";
 /*******************************************************************************
 *									       *
 * Copyright (C) 1999 Mark Edel						       *
@@ -6,7 +6,8 @@ static const char CVSID[] = "$Id: selection.c,v 1.25 2003/05/09 17:43:48 edg Exp
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
@@ -35,6 +36,7 @@ static const char CVSID[] = "$Id: selection.c,v 1.25 2003/05/09 17:43:48 edg Exp
 #include "file.h"
 #include "window.h"
 #include "menu.h"
+#include "preferences.h"
 #include "server.h"
 #include "../util/DialogF.h"
 #include "../util/fileUtils.h"
@@ -322,7 +324,8 @@ static void fileCB(Widget widget, WindowInfo *window, Atom *sel,
         XBell(TheDisplay, 0);
 	return;
     }	
-    EditExistingFile(WindowList, filename, pathname, 0, NULL, False, NULL);
+    EditExistingFile(window, filename, 
+            pathname, 0, NULL, False, NULL, GetPrefOpenInTab(), False);
 #elif defined(USE_MOTIF_GLOB)
     { char **nameList = NULL;
       int i, nFiles = 0, maxFiles = 30;
@@ -338,8 +341,8 @@ static void fileCB(Widget widget, WindowInfo *window, Atom *sel,
 	      XBell(TheDisplay, 0);
 	  }
         else {
-    	      EditExistingFile(WindowList, filename, pathname, 0, NULL, 
-	                       False, NULL);
+    	      EditExistingFile(window, filename, pathname, 0, 
+	              NULL, False, NULL, GetPrefOpenInTab(), False);
 	  }
       }
       for (i=0; i<nFiles; i++) {
@@ -356,8 +359,9 @@ static void fileCB(Widget widget, WindowInfo *window, Atom *sel,
 	  if (ParseFilename(globbuf.gl_pathv[i], filename, pathname) != 0)
 	      XBell(TheDisplay, 0);
 	  else
-    	      EditExistingFile(WindowList, filename, pathname, 0, NULL,
-	                       False, NULL);
+    	      EditExistingFile(GetPrefOpenInTab()? window : NULL, 
+	              filename, pathname, 0, NULL, False, NULL, 
+		      GetPrefOpenInTab(), False);
       }
       globfree(&globbuf);
     }
