@@ -1,35 +1,62 @@
+static const char CVSID[] = "$Id: shell.c,v 1.35 2004/09/02 08:49:56 edg Exp $";
 /*******************************************************************************
 *									       *
 * shell.c -- Nirvana Editor shell command execution			       *
 *									       *
-* Copyright (c) 1991-1997 Universities Research Association, Inc.	       *
-* All rights reserved.							       *
+* Copyright (C) 1999 Mark Edel						       *
+*									       *
+* This is free software; you can redistribute it and/or modify it under the    *
+* terms of the GNU General Public License as published by the Free Software    *
+* Foundation; either version 2 of the License, or (at your option) any later   *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
 * 									       *
-* This material resulted from work developed under a Government Contract and   *
-* is subject to the following license:  The Government retaFins a paid-up,     *
-* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
-* works, perform publicly and display publicly by or for the Government,       *
-* including the right to distribute to other Government contractors.  Neither  *
-* the United States nor the United States Department of Energy, nor any of     *
-* their employees, makes any warranty, express or implied, or assumes any      *
-* legal liability or responsibility for the accuracy, completeness, or         *
-* usefulness of any information, apparatus, product, or process disclosed, or  *
-* represents that its use would not infringe privately owned rights.           *
-*                                        				       *
-* Fermilab Nirvana GUI Library						       *
+* This software is distributed in the hope that it will be useful, but WITHOUT *
+* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
+* for more details.							       *
+* 									       *
+* You should have received a copy of the GNU General Public License along with *
+* software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
+* Place, Suite 330, Boston, MA  02111-1307 USA		                       *
+*									       *
+* Nirvana Text Editor	    						       *
 * December, 1993							       *
 *									       *
 * Written by Mark Edel							       *
 *									       *
 *******************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "shell.h"
+#include "textBuf.h"
+#include "text.h"
+#include "nedit.h"
+#include "window.h"
+#include "preferences.h"
+#include "file.h"
+#include "macro.h"
+#include "interpret.h"
+#include "../util/DialogF.h"
+#include "../util/misc.h"
+#include "menu.h"
+
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/param.h>
-#include <sys/wait.h>
+#include <string.h>
+#include <signal.h>
 #include <sys/types.h>
+#ifndef __MVS__
+#include <sys/param.h>
+#endif
+#include <sys/wait.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <ctype.h>
+#include <errno.h>
 #ifdef notdef
 #ifdef IBM
 #define NBBY 8
@@ -37,29 +64,23 @@
 #endif
 #include <time.h>
 #endif
-#include <errno.h>
-#include <signal.h>
+#ifdef __EMX__
+#include <process.h>
+#endif
+
 #include <Xm/Xm.h>
 #include <Xm/MessageB.h>
 #include <Xm/Text.h>
 #include <Xm/Form.h>
 #include <Xm/PushBG.h>
-#include "../util/DialogF.h"
-#include "../util/misc.h"
-#include "textBuf.h"
-#include "text.h"
-#include "nedit.h"
-#include "window.h"
-#include "preferences.h"
-#include "file.h"
-#include "shell.h"
-#include "macro.h"
-#include "interpret.h"
+
+#ifdef HAVE_DEBUG_H
+#include "../debug.h"
+#endif
+
 
 /* Tuning parameters */
 #define IO_BUF_SIZE 4096	/* size of buffers for collecting cmd output */
-#define MAX_SHELL_CMD_LEN 1024	/* max length of a shell command (should be
-				   eliminated, but substitutePercent needs) */
 #define MAX_OUT_DIALOG_ROWS 30	/* max height of dialog for command output */
 #define MAX_OUT_DIALOG_COLS 80	/* max width of dialog for command output */
 #define OUTPUT_FLUSH_FREQ 1000	/* how often (msec) to flush output buffers
@@ -100,14 +121,14 @@ typedef struct {
     char fromMacro;
 } shellCmdInfo;
 
-static void issueCommand(WindowInfo *window, char *command, char *input,
+static void issueCommand(WindowInfo *window, const char *command, char *input,
 	int inputLen, int flags, Widget textW, int replaceLeft,
 	int replaceRight, int fromMacro);
 static void stdoutReadProc(XtPointer clientData, int *source, XtInputId *id);
 static void stderrReadProc(XtPointer clientData, int *source, XtInputId *id);
 static void stdinWriteProc(XtPointer clientData, int *source, XtInputId *id);
 static void finishCmdExecution(WindowInfo *window, int terminatedOnError);
-static pid_t forkCommand(Widget parent, char *command, char *cmdDir,
+static pid_t forkCommand(Widget parent, const char *command, const char *cmdDir,
 	int *stdinFD, int *stdoutFD, int *stderrFD);
 static void addOutput(buffer **bufList, buffer *buf);
 static char *coalesceOutput(buffer **bufList, int *length);
@@ -115,19 +136,24 @@ static void freeBufList(buffer **bufList);
 static void removeTrailingNewlines(char *string);
 static void createOutputDialog(Widget parent, char *text);
 static void destroyOutDialogCB(Widget w, XtPointer callback, XtPointer closure);
-static void measureText(char *text, int wrapWidth, int *rows, int *cols);
+static void measureText(char *text, int wrapWidth, int *rows, int *cols,
+	int *wrapped);
 static void truncateString(char *string, int length);
-static int substitutePercent(char *outStr, char *inStr, char *subsStr,
-	int outLen);
 static void bannerTimeoutProc(XtPointer clientData, XtIntervalId *id);
 static void flushTimeoutProc(XtPointer clientData, XtIntervalId *id);
+static void safeBufReplace(textBuffer *buf, int *start, int *end, 
+	const char *text);
+static char *shellCommandSubstitutes(const char *inStr, const char *fileStr,
+	const char *lineStr);
+static int shellSubstituter(char *outStr, const char *inStr, const char *fileStr,
+	const char *lineStr, int outLen, int predictOnly);
 
 /*
 ** Filter the current selection through shell command "command".  The selection
 ** is removed, and replaced by the output from the command execution.  Failed
 ** command status and output to stderr are presented in dialog form.
 */
-void FilterSelection(WindowInfo *window, char *command, int fromMacro)
+void FilterSelection(WindowInfo *window, const char *command, int fromMacro)
 {
     int left, right, textLen;
     char *text;
@@ -158,12 +184,15 @@ void FilterSelection(WindowInfo *window, char *command, int fromMacro)
 
 /*
 ** Execute shell command "command", depositing the result at the current
-** insert position or in the current selection if if the window has a
+** insert position or in the current selection if the window has a
 ** selection.
 */
-void ExecShellCommand(WindowInfo *window, char *command, int fromMacro)
+void ExecShellCommand(WindowInfo *window, const char *command, int fromMacro)
 {
     int left, right, flags = 0;
+    char *subsCommand, fullName[MAXPATHLEN];
+    int pos, line, column;
+    char lineNumber[11];
 
     /* Can't do two shell commands at once in the same window */
     if (window->shellCmdData != NULL) {
@@ -172,21 +201,41 @@ void ExecShellCommand(WindowInfo *window, char *command, int fromMacro)
     }
     
     /* get the selection or the insert position */
+    pos = TextGetCursorPos(window->lastFocus);
     if (GetSimpleSelection(window->buffer, &left, &right))
     	flags = ACCUMULATE | REPLACE_SELECTION;
     else
-    	left = right = TextGetCursorPos(window->lastFocus);
+    	left = right = pos;
     
+    /* Substitute the current file name for % and the current line number
+       for # in the shell command */
+    strcpy(fullName, window->path);
+    strcat(fullName, window->filename);
+    TextPosToLineAndCol(window->lastFocus, pos, &line, &column);
+    sprintf(lineNumber, "%d", line);
+    
+    subsCommand = shellCommandSubstitutes(command, fullName, lineNumber);
+    if (subsCommand == NULL)
+    {
+        DialogF(DF_ERR, window->shell, 1, "Shell Command",
+                "Shell command is too long due to\n"
+                "filename substitutions with '%%' or\n"
+                "line number substitutions with '#'", "OK");
+        return;
+    }
+
     /* issue the command */
-    issueCommand(window, command, NULL, 0, flags, window->lastFocus, left,
+    issueCommand(window, subsCommand, NULL, 0, flags, window->lastFocus, left,
 	    right, fromMacro);
+    free(subsCommand);
 }
 
 /*
 ** Execute shell command "command", on input string "input", depositing the
 ** in a macro string (via a call back to ReturnShellCommandOutput).
 */
-void ShellCmdToMacroString(WindowInfo *window, char *command, char *input)
+void ShellCmdToMacroString(WindowInfo *window, const char *command,
+   const char *input)
 {
     char *inputCopy;
     
@@ -207,6 +256,9 @@ void ExecCursorLine(WindowInfo *window, int fromMacro)
 {
     char *cmdText;
     int left, right, insertPos;
+    char *subsCommand, fullName[MAXPATHLEN];
+    int pos, line, column;
+    char lineNumber[11];
 
     /* Can't do two shell commands at once in the same window */
     if (window->shellCmdData != NULL) {
@@ -215,8 +267,9 @@ void ExecCursorLine(WindowInfo *window, int fromMacro)
     }
 
     /* get all of the text on the line with the insert position */
+    pos = TextGetCursorPos(window->lastFocus);
     if (!GetSimpleSelection(window->buffer, &left, &right)) {
-	left = right = TextGetCursorPos(window->lastFocus);
+	left = right = pos;
 	left = BufStartOfLine(window->buffer, left);
 	right = BufEndOfLine(window->buffer, right);
 	insertPos = right;
@@ -228,9 +281,27 @@ void ExecCursorLine(WindowInfo *window, int fromMacro)
     /* insert a newline after the entire line */
     BufInsert(window->buffer, insertPos, "\n");
 
+    /* Substitute the current file name for % and the current line number
+       for # in the shell command */
+    strcpy(fullName, window->path);
+    strcat(fullName, window->filename);
+    TextPosToLineAndCol(window->lastFocus, pos, &line, &column);
+    sprintf(lineNumber, "%d", line);
+    
+    subsCommand = shellCommandSubstitutes(cmdText, fullName, lineNumber);
+    if (subsCommand == NULL)
+    {
+        DialogF(DF_ERR, window->shell, 1, "Shell Command",
+                "Shell command is too long due to\n"
+                "filename substitutions with '%%' or\n"
+                "line number substitutions with '#'", "OK");
+        return;
+    }
+
     /* issue the command */
-    issueCommand(window, cmdText, NULL, 0, 0, window->lastFocus, insertPos+1,
+    issueCommand(window, subsCommand, NULL, 0, 0, window->lastFocus, insertPos+1,
 	    insertPos+1, fromMacro);
+    free(subsCommand);
     XtFree(cmdText);
 }
 
@@ -239,13 +310,16 @@ void ExecCursorLine(WindowInfo *window, int fromMacro)
 ** output destination, save first and load after) in the shell commands
 ** menu.
 */
-void DoShellMenuCmd(WindowInfo *window, char *command, int input, int output,
+void DoShellMenuCmd(WindowInfo *window, const char *command,
+        int input, int output,
 	int outputReplacesInput, int saveFirst, int loadAfter, int fromMacro) 
 {
     int flags = 0;
     char *text;
-    char subsCommand[MAX_SHELL_CMD_LEN], fullName[MAXPATHLEN];
+    char *subsCommand, fullName[MAXPATHLEN];
     int left, right, textLen;
+    int pos, line, column;
+    char lineNumber[11];
     WindowInfo *inWindow = window;
     Widget outWidget;
 
@@ -255,23 +329,31 @@ void DoShellMenuCmd(WindowInfo *window, char *command, int input, int output,
     	return;
     }
 
-    /* Substitute the current file name for % in the shell command */
+    /* Substitute the current file name for % and the current line number
+       for # in the shell command */
     strcpy(fullName, window->path);
     strcat(fullName, window->filename);
-    if (!substitutePercent(subsCommand, command, fullName,
-    	    MAX_SHELL_CMD_LEN)) {
-    	DialogF(DF_ERR, window->shell, 1,
-	   "Shell command is too long due to\nfilename substitutions with '%%'",
-	    "OK");
-	return;
+    pos = TextGetCursorPos(window->lastFocus);
+    TextPosToLineAndCol(window->lastFocus, pos, &line, &column);
+    sprintf(lineNumber, "%d", line);
+    
+    subsCommand = shellCommandSubstitutes(command, fullName, lineNumber);
+    if (subsCommand == NULL)
+    {
+        DialogF(DF_ERR, window->shell, 1, "Shell Command",
+                "Shell command is too long due to\n"
+                "filename substitutions with '%%' or\n"
+                "line number substitutions with '#'", "OK");
+        return;
     }
-    	
+
     /* Get the command input as a text string.  If there is input, errors
       shouldn't be mixed in with output, so set flags to ERROR_DIALOGS */
     if (input == FROM_SELECTION) {
 	text = BufGetSelectionText(window->buffer);
 	if (*text == '\0') {
     	    XtFree(text);
+            free(subsCommand);
     	    XBell(TheDisplay, 0);
     	    return;
     	}
@@ -305,10 +387,11 @@ void DoShellMenuCmd(WindowInfo *window, char *command, int input, int output,
 	flags |= OUTPUT_TO_DIALOG;
     	left = right = 0;
     } else if (output == TO_NEW_WINDOW) {
-    	EditNewFile();
+    	EditNewFile(GetPrefOpenInTab()?inWindow:NULL, NULL, False, NULL, window->path);
     	outWidget = WindowList->textArea;
 	inWindow = WindowList;
     	left = right = 0;
+	CheckCloseDim();
     } else { /* TO_SAME_WINDOW */
     	outWidget = window->lastFocus;
     	if (outputReplacesInput && input != FROM_NONE) {
@@ -339,6 +422,7 @@ void DoShellMenuCmd(WindowInfo *window, char *command, int input, int output,
     	if (!SaveWindow(window)) {
     	    if (input != FROM_NONE)
     		XtFree(text);
+            free(subsCommand);
     	    return;
 	}
     }
@@ -351,6 +435,7 @@ void DoShellMenuCmd(WindowInfo *window, char *command, int input, int output,
     /* issue the command */
     issueCommand(inWindow, subsCommand, text, textLen, flags, outWidget, left,
 	    right, fromMacro);
+    free(subsCommand);
 }
 
 /*
@@ -389,7 +474,7 @@ void AbortShellCommand(WindowInfo *window)
 ** REPLACE_SELECTION, ERROR_DIALOGS, and OUTPUT_TO_STRING can only be used
 ** along with ACCUMULATE (these operations can't be done incrementally).
 */
-static void issueCommand(WindowInfo *window, char *command, char *input,
+static void issueCommand(WindowInfo *window, const char *command, char *input,
 	int inputLen, int flags, Widget textW, int replaceLeft,
 	int replaceRight, int fromMacro)
 {
@@ -415,7 +500,7 @@ static void issueCommand(WindowInfo *window, char *command, char *input,
     
     /* enable the cancel menu item */
     if (!fromMacro)
-    	XtSetSensitive(window->cancelShellItem, True);
+    	SetSensitive(window, window->cancelShellItem, True);
 
     /* fork the subprocess and issue the command */
     childPid = forkCommand(window->shell, command, window->path, &stdinFD,
@@ -583,7 +668,14 @@ static void stdinWriteProc(XtPointer clientData, int *source, XtInputId *id)
 
     nWritten = write(cmdData->stdinFD, cmdData->inPtr, cmdData->inLength);
     if (nWritten == -1) {
-    	if (errno != EWOULDBLOCK && errno != EAGAIN) {
+	if (errno == EPIPE) {
+	    /* Just shut off input to broken pipes.  User is likely feeding
+	       it to a command which does not take input */
+	    XtRemoveInput(cmdData->stdinInputID);
+	    cmdData->stdinInputID = 0;
+    	    close(cmdData->stdinFD);
+    	    cmdData->inPtr = NULL;
+    	} else if (errno != EWOULDBLOCK && errno != EAGAIN) {
     	    perror("NEdit: Write to shell command failed");
     	    finishCmdExecution(window, True);
     	}
@@ -614,6 +706,22 @@ static void bannerTimeoutProc(XtPointer clientData, XtIntervalId *id)
     cmdData->bannerTimeoutID = 0;
 }
 
+/*
+** Buffer replacement wrapper routine to be used for inserting output from 
+** a command into the buffer, which takes into account that the buffer may 
+** have been shrunken by the user (eg, by Undo). If necessary, the starting
+** and ending positions (part of the state of the command) are corrected.
+*/
+static void safeBufReplace(textBuffer *buf, int *start, int *end, 
+	const char *text)
+{
+    if (*start > buf->length)
+	*start = buf->length;
+    if (*end > buf->length)
+	*end = buf->length;
+    BufReplace(buf, *start, *end, text);
+}
+
 /*
 ** Timer proc for flushing output buffers periodically when the process
 ** takes too long.
@@ -633,7 +741,7 @@ static void flushTimeoutProc(XtPointer clientData, XtIntervalId *id)
     outText = coalesceOutput(&cmdData->outBufs, &len);
     if (len != 0) {
 	if (BufSubstituteNullChars(outText, len, buf)) {
-	    BufReplace(buf, cmdData->leftPos, cmdData->rightPos, outText);
+	    safeBufReplace(buf, &cmdData->leftPos, &cmdData->rightPos, outText);
 	    TextSetCursorPos(cmdData->textW, cmdData->leftPos+strlen(outText));
 	    cmdData->leftPos += len;
 	    cmdData->rightPos = cmdData->leftPos;
@@ -661,7 +769,7 @@ static void finishCmdExecution(WindowInfo *window, int terminatedOnError)
     textBuffer *buf;
     int status, failure, errorReport, reselectStart, outTextLen, errTextLen;
     int resp, cancel = False, fromMacro = cmdData->fromMacro;
-    char *outText, *errText;
+    char *outText, *errText = NULL;
 
     /* Cancel any pending i/o on the file descriptors */
     if (cmdData->stdoutInputID != 0)
@@ -691,7 +799,7 @@ static void finishCmdExecution(WindowInfo *window, int terminatedOnError)
     /* Clean up waiting-for-shell-command-to-complete mode */
     if (!cmdData->fromMacro) {
 	EndWait(window->shell);
-	XtSetSensitive(window->cancelShellItem, False);
+	SetSensitive(window, window->cancelShellItem, False);
 	if (cmdData->bannerIsUp)
     	    ClearModeMessage(window);
     }
@@ -700,6 +808,7 @@ static void finishCmdExecution(WindowInfo *window, int terminatedOnError)
     if (terminatedOnError) {
 	freeBufList(&cmdData->outBufs);
 	freeBufList(&cmdData->errBufs);
+    	waitpid(cmdData->childPid, &status, 0);
 	goto cmdDone;
     }
 
@@ -715,33 +824,40 @@ static void finishCmdExecution(WindowInfo *window, int terminatedOnError)
     /* Present error and stderr-information dialogs.  If a command returned
        error output, or if the process' exit status indicated failure,
        present the information to the user. */
-    if (cmdData->flags & ERROR_DIALOGS) {
-	failure = WIFEXITED(status) && WEXITSTATUS(status) != 0;
-	errorReport = *errText != '\0';
-	if (failure && errorReport) {
-    	    removeTrailingNewlines(errText);
-    	    truncateString(errText, DF_MAX_MSG_LENGTH);
-    	    resp = DialogF(DF_WARN, window->shell, 2, "%s",
-    	    	    "Cancel", "Proceed", errText);
-    	    cancel = resp == 1;
-	} else if (failure) {
-    	    truncateString(outText, DF_MAX_MSG_LENGTH-70);
-    	    resp = DialogF(DF_WARN, window->shell, 2,
-    	       "Command reported failed exit status.\nOutput from command:\n%s",
-    		    "Cancel", "Proceed", outText);
-    	    cancel = resp == 1;
-	} else if (errorReport) {
-    	    removeTrailingNewlines(errText);
-    	    truncateString(errText, DF_MAX_MSG_LENGTH);
-    	    resp = DialogF(DF_INF, window->shell, 2, "%s",
-    	    	    "Proceed", "Cancel", errText);
-    	    cancel = resp == 2;
-	}
-	XtFree(errText);
-	if (cancel) {
-	    XtFree(outText);
-    	    goto cmdDone;
-	}
+    if (cmdData->flags & ERROR_DIALOGS)
+    {
+        failure = WIFEXITED(status) && WEXITSTATUS(status) != 0;
+        errorReport = *errText != '\0';
+
+        if (failure && errorReport)
+        {
+            removeTrailingNewlines(errText);
+            truncateString(errText, DF_MAX_MSG_LENGTH);
+            resp = DialogF(DF_WARN, window->shell, 2, "Warning", "%s", "Cancel",
+                    "Proceed", errText);
+            cancel = resp == 1;
+        } else if (failure)
+        {
+            truncateString(outText, DF_MAX_MSG_LENGTH-70);
+            resp = DialogF(DF_WARN, window->shell, 2, "Command Failure",
+                    "Command reported failed exit status.\n"
+                    "Output from command:\n%s", "Cancel", "Proceed", outText);
+            cancel = resp == 1;
+        } else if (errorReport)
+        {
+            removeTrailingNewlines(errText);
+            truncateString(errText, DF_MAX_MSG_LENGTH);
+            resp = DialogF(DF_INF, window->shell, 2, "Information", "%s",
+                    "Proceed", "Cancel", errText);
+            cancel = resp == 2;
+        }
+
+        XtFree(errText);
+        if (cancel)
+        {
+            XtFree(outText);
+            goto cmdDone;
+        }
     }
     
     /* If output is to a dialog, present the dialog.  Otherwise insert the
@@ -766,7 +882,7 @@ static void finishCmdExecution(WindowInfo *window, int terminatedOnError)
 	    if (reselectStart != -1)
 	    	BufSelect(buf, reselectStart, reselectStart + strlen(outText));
 	} else {
-	    BufReplace(buf, cmdData->leftPos, cmdData->rightPos, outText);
+	    safeBufReplace(buf, &cmdData->leftPos, &cmdData->rightPos, outText);
 	    TextSetCursorPos(cmdData->textW, cmdData->leftPos+strlen(outText));
 	}
     }
@@ -792,24 +908,28 @@ cmdDone:
 ** stderr.  The function value returns the pid of the new subprocess, or -1
 ** if an error occured.
 */
-static pid_t forkCommand(Widget parent, char *command, char *cmdDir,
+static pid_t forkCommand(Widget parent, const char *command, const char *cmdDir,
 	int *stdinFD, int *stdoutFD, int *stderrFD)
 {
     int childStdoutFD, childStdinFD, childStderrFD, pipeFDs[2];
     int dupFD;
     pid_t childPid;
     
+    /* Ignore SIGPIPE signals generated when user attempts to provide
+       input for commands which don't take input */
+    signal(SIGPIPE, SIG_IGN);
+    
     /* Create pipes to communicate with the sub process.  One end of each is
        returned to the caller, the other half is spliced to stdin, stdout
        and stderr in the child process */
     if (pipe(pipeFDs) != 0) {
-    	perror("Nedit: Internal error (opening stdout pipe)");
+    	perror("NEdit: Internal error (opening stdout pipe)");
         return -1;
     }
     *stdoutFD = pipeFDs[0];
     childStdoutFD = pipeFDs[1];
     if (pipe(pipeFDs) != 0) {
-    	perror("Nedit: Internal error (opening stdin pipe)");
+    	perror("NEdit: Internal error (opening stdin pipe)");
         return -1;
     }
     *stdinFD = pipeFDs[1];
@@ -818,7 +938,7 @@ static pid_t forkCommand(Widget parent, char *command, char *cmdDir,
     	childStderrFD = childStdoutFD;
     else {
 	if (pipe(pipeFDs) != 0) {
-    	    perror("Nedit: Internal error (opening stdin pipe)");
+    	    perror("NEdit: Internal error (opening stdin pipe)");
             return -1;
         }
 	*stderrFD = pipeFDs[0];
@@ -857,10 +977,18 @@ static pid_t forkCommand(Widget parent, char *command, char *cmdDir,
  	dupFD = dup2(childStderrFD, fileno(stderr));
 	if (dupFD == -1)
 	    perror("dup of stderr failed");
+
+        /* now close the original child end of the pipes
+           (we now have the 0, 1 and 2 descriptors in their place) */
+        close(childStdinFD);
+        close(childStdoutFD);
+        close(childStderrFD);
 	
 	/* make this process the leader of a new process group, so the sub
 	   processes can be killed, if necessary, with a killpg call */
+#ifndef __EMX__  /* OS/2 doesn't have this */
 	setsid();
+#endif
       
        /* change the current working directory to the directory of the current
 	  file. */ 
@@ -869,18 +997,20 @@ static pid_t forkCommand(Widget parent, char *command, char *cmdDir,
 	       perror("chdir to directory of current file failed");
      
 	/* execute the command using the shell specified by preferences */
-	execl(GetPrefShell(), GetPrefShell(), "-c", command, 0);
+	execl(GetPrefShell(), GetPrefShell(), "-c", command, (char *)0);
 
 	/* if we reach here, execl failed */
 	fprintf(stderr, "Error starting shell: %s\n", GetPrefShell());
-	exit(1);
+	exit(EXIT_FAILURE);
     }
     
     /* Parent process context, check if fork succeeded */
     if (childPid == -1)
-    	DialogF(DF_ERR, parent, 1,
-		"Error starting shell command process\n(fork failed)",
-		"Dismiss");
+    {
+        DialogF(DF_ERR, parent, 1, "Shell Command",
+                "Error starting shell command process\n(fork failed)",
+                "OK");
+    }
 
     /* close the child ends of the pipes */
     close(childStdinFD);
@@ -973,12 +1103,12 @@ static void removeTrailingNewlines(char *string)
 static void createOutputDialog(Widget parent, char *text)
 {
     Arg al[50];
-    int ac, rows, cols, hasScrollBar;
+    int ac, rows, cols, hasScrollBar, wrapped;
     Widget form, textW, button;
     XmString st1;
 
     /* measure the width and height of the text to determine size for dialog */
-    measureText(text, MAX_OUT_DIALOG_COLS, &rows, &cols);
+    measureText(text, MAX_OUT_DIALOG_COLS, &rows, &cols, &wrapped);
     if (rows > MAX_OUT_DIALOG_ROWS) {
     	rows = MAX_OUT_DIALOG_ROWS;
     	hasScrollBar = True;
@@ -988,18 +1118,25 @@ static void createOutputDialog(Widget parent, char *text)
     	cols = MAX_OUT_DIALOG_COLS;
     if (cols == 0)
     	cols = 1;
-    
+    /* Without completely emulating Motif's wrapping algorithm, we can't
+       be sure that we haven't underestimated the number of lines in case
+       a line has wrapped, so let's assume that some lines could be obscured
+       */
+    if (wrapped)
+	hasScrollBar = True;
     ac = 0;
-    form = XmCreateFormDialog(parent, "shellOutForm", al, ac);
+    form = CreateFormDialog(parent, "shellOutForm", al, ac);
 
     ac = 0;
-    XtSetArg(al[ac], XmNlabelString, st1=MKSTRING("Dismiss")); ac++;
+    XtSetArg(al[ac], XmNlabelString, st1=MKSTRING("OK")); ac++;
+    XtSetArg(al[ac], XmNmarginWidth, BUTTON_WIDTH_MARGIN); ac++;
     XtSetArg(al[ac], XmNhighlightThickness, 0);  ac++;
     XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM);  ac++;
     XtSetArg(al[ac], XmNtopAttachment, XmATTACH_NONE);  ac++;
-    button = XmCreatePushButtonGadget(form, "dismiss", al, ac);
+    button = XmCreatePushButtonGadget(form, "ok", al, ac);
     XtManageChild(button);
-    XtVaSetValues(form, XmNdefaultButton, button, 0);
+    XtVaSetValues(form, XmNdefaultButton, button, NULL);
+    XtVaSetValues(form, XmNcancelButton, button, NULL);
     XmStringFree(st1);
     XtAddCallback(button, XmNactivateCallback, destroyOutDialogCB,
     	    XtParent(form));
@@ -1023,9 +1160,10 @@ static void createOutputDialog(Widget parent, char *text)
     XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM);  ac++;
     XtSetArg(al[ac], XmNbottomWidget, button);  ac++;
     textW = XmCreateScrolledText(form, "outText", al, ac);
+    AddMouseWheelSupport(textW);
     XtManageChild(textW);
     
-    XtVaSetValues(XtParent(form), XmNtitle, "Output from Command", 0);
+    XtVaSetValues(XtParent(form), XmNtitle, "Output from Command", NULL);
     ManageDialogCenteredOnPointer(form);
 }
 
@@ -1041,23 +1179,52 @@ static void destroyOutDialogCB(Widget w, XtPointer callback, XtPointer closure)
 ** Measure the width and height of a string of text.  Assumes 8 character
 ** tabs.  wrapWidth specifies a number of columns at which text wraps.
 */
-static void measureText(char *text, int wrapWidth, int *rows, int *cols)
+static void measureText(char *text, int wrapWidth, int *rows, int *cols,
+	int *wrapped)
 {
-    int maxCols = 0, line = 1, col = 0;
+    int maxCols = 0, line = 1, col = 0, wrapCol;
     char *c;
     
+    *wrapped = 0;
     for (c=text; *c!='\0'; c++) {
-    	if (*c=='\n' || col > wrapWidth) {
-    	    line++;
-    	    col = 0;
-    	} else {
-    	    if (*c == '\t')
-    		col += 8 - (col % 8);
-    	    else
-    		col++;
-    	    if (col > maxCols)
-    	    	maxCols = col;
-    	}
+    	if (*c=='\n') {
+	    line++;
+	    col = 0;
+	    continue;
+	} 
+
+	if (*c == '\t') {
+	    col += 8 - (col % 8);
+	    wrapCol = 0; /* Tabs at end of line are not drawn when wrapped */
+	} else if (*c == ' ') {
+	    col++;
+	    wrapCol = 0; /* Spaces at end of line are not drawn when wrapped */
+	} else {
+	    col++;
+	    wrapCol = 1;
+	}
+
+	/* Note: there is a small chance that the number of lines is
+	   over-estimated when a line ends with a space or a tab (ie, followed
+           by a newline) and that whitespace crosses the boundary, because
+           whitespace at the end of a line does not cause wrapping. Taking
+           this into account is very hard, but an over-estimation is harmless.
+           The worst that can happen is that some extra blank lines are shown
+           at the end of the dialog (in contrast to an under-estimation, which
+           could make the last lines invisible).
+           On the other hand, without emulating Motif's wrapping algorithm
+           completely, we can't be sure that we don't underestimate the number
+           of lines (Motif uses word wrap, and this counting algorithm uses
+           character wrap). Therefore, we remember whether there is a line
+           that has wrapped. In that case we allways install a scroll bar.
+	   */
+	if (col > wrapWidth) {
+	    line++;
+	    *wrapped = 1;
+	    col = wrapCol;
+	} else if (col > maxCols) {
+	    maxCols = col;
+	}
     }
     *rows = line;
     *cols = maxCols;
@@ -1070,37 +1237,110 @@ static void measureText(char *text, int wrapWidth, int *rows, int *cols)
 */
 static void truncateString(char *string, int length)
 {
-    if (strlen(string) > length)
+    if ((int)strlen(string) > length)
 	memcpy(&string[length-3], "...", 4);
 }
 
 /*
-** Substitute the string subsStr in inStr wherever % appears, storing the
-** result in outStr.  Returns False if the resulting string would be
-** longer than outLen
+** Substitute the string fileStr in inStr wherever % appears and
+** lineStr in inStr wherever # appears, storing the
+** result in outStr. If predictOnly is non-zero, the result string length
+** is predicted without creating the string. Returns the length of the result
+** string or -1 in case of an error.
+**
 */
-static int substitutePercent(char *outStr, char *inStr, char *subsStr,
-	int outLen)
+static int shellSubstituter(char *outStr, const char *inStr, const char *fileStr,
+        const char *lineStr, int outLen, int predictOnly)
 {
-    char *inChar, *outChar, *c;
-    
+    const char *inChar;
+    char *outChar = NULL;
+    int outWritten = 0;
+    int fileLen, lineLen;
+
     inChar = inStr;
-    outChar = outStr;
-    while (*inChar != '\0') {
-    	if (*inChar == '%') {
-    	    if (*(inChar+1) == '%') {
-    	    	inChar += 2;
-    	    	*outChar++ = '%';
-    	    } else {
-    		for (c=subsStr; *c!='\0'; c++)
-    	    	    *outChar++ = *c;
-    		inChar++;
-    	    }
-    	} else
-    	    *outChar++ = *inChar++;
-    	if (outChar - outStr >= outLen)
-    	    return False;
+    if (!predictOnly) {
+        outChar = outStr;
     }
-    *outChar = '\0';
-    return True;
+    fileLen = strlen(fileStr);
+    lineLen = strlen(lineStr);
+
+    while (*inChar != '\0') {
+
+        if (!predictOnly && outWritten >= outLen) {
+            return(-1);
+        }
+
+        if (*inChar == '%') {
+            if (*(inChar + 1) == '%') {
+                inChar += 2;
+                if (!predictOnly) {
+                    *outChar++ = '%';
+                }
+                outWritten++;
+            } else  {
+                if (!predictOnly) {
+                    if (outWritten + fileLen >= outLen) {
+                        return(-1);
+                    }
+                    strncpy(outChar, fileStr, fileLen);
+                    outChar += fileLen;
+                }
+                outWritten += fileLen;
+                inChar++;
+            }
+        } else if (*inChar == '#') {
+            if (*(inChar + 1) == '#') {
+                inChar += 2;
+                if (!predictOnly) {
+                    *outChar++ = '#';
+                }
+                outWritten++;
+            } else  {
+                if (!predictOnly) {
+                    if (outWritten + lineLen >= outLen) {
+                        return(-1);
+                    }
+                    strncpy(outChar, lineStr, lineLen);
+                    outChar += lineLen;
+                }
+                outWritten += lineLen;
+                inChar++;
+            }
+        } else {
+            if (!predictOnly) {
+                *outChar++ = *inChar;
+            }
+            inChar++;
+            outWritten++;
+        }
+    }
+
+    if (!predictOnly) {
+        if (outWritten >= outLen) {
+            return(-1);
+        }
+        *outChar = '\0';
+    }
+    ++outWritten;
+    return(outWritten);
+}
+
+static char *shellCommandSubstitutes(const char *inStr, const char *fileStr,
+        const char *lineStr)
+{
+    int cmdLen;
+    char *subsCmdStr = NULL;
+
+    cmdLen = shellSubstituter(NULL, inStr, fileStr, lineStr, 0, 1);
+    if (cmdLen >= 0) {
+        subsCmdStr = malloc(cmdLen);
+        if (subsCmdStr) {
+            cmdLen = shellSubstituter(subsCmdStr, inStr, fileStr, lineStr, cmdLen, 0);
+            if (cmdLen < 0) {
+                free(subsCmdStr);
+                subsCmdStr = NULL;
+            }
+        }
+    }
+    return(subsCmdStr);
 }
