@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: shift.c,v 1.8 2001/08/14 08:37:16 jlous Exp $";
+static const char CVSID[] = "$Id: shift.c,v 1.16 2004/07/21 11:32:05 yooden Exp $";
 /*******************************************************************************
 *									       *
 * shift.c -- Nirvana Editor built-in filter commands			       *
@@ -8,7 +8,8 @@ static const char CVSID[] = "$Id: shift.c,v 1.8 2001/08/14 08:37:16 jlous Exp $"
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
@@ -25,6 +26,20 @@ static const char CVSID[] = "$Id: shift.c,v 1.8 2001/08/14 08:37:16 jlous Exp $"
 * Written by Mark Edel							       *
 *									       *
 *******************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "shift.h"
+#include "textBuf.h"
+#include "text.h"
+#include "nedit.h"
+#include "window.h"
+
+#include <string.h>
+#include <limits.h>
+#include <ctype.h>
 #ifdef VMS
 #include "../util/VMSparam.h"
 #else
@@ -32,15 +47,13 @@ static const char CVSID[] = "$Id: shift.c,v 1.8 2001/08/14 08:37:16 jlous Exp $"
 #include <sys/param.h>
 #endif
 #endif /*VMS*/
-#include <limits.h>
-#include <ctype.h>
+
 #include <Xm/Xm.h>
-#include "../util/DialogF.h"
-#include "textBuf.h"
-#include "text.h"
-#include "nedit.h"
-#include "shift.h"
-#include "window.h"
+
+#ifdef HAVE_DEBUG_H
+#include "../debug.h"
+#endif
+
 
 static void shiftRect(WindowInfo *window, int direction, int byTab,
 	int selStart, int selEnd, int rectStart, int rectEnd);
@@ -54,10 +67,10 @@ static char *fillParagraphs(char *text, int rightMargin, int tabDist,
 static char *fillParagraph(char *text, int leftMargin, int firstLineIndent,
 	int rightMargin, int tabDist, int allowTabs, char nullSubsChar,
 	int *filledLen);
-char *makeIndentString(int indent, int tabDist, int allowTabs, int *nChars);
+static char *makeIndentString(int indent, int tabDist, int allowTabs, int *nChars);
 static int atTabStop(int pos, int tabDist);
 static int nextTab(int pos, int tabDist);
-static int countLines(char *text);
+static int countLines(const char *text);
 static int findParagraphStart(textBuffer *buf, int startPos);
 static int findParagraphEnd(textBuffer *buf, int startPos);
 
@@ -251,10 +264,13 @@ void FillSelection(WindowInfo *window)
        by measuring the text and querying the window's wrap margin (or width) */
     if (hasSelection && isRect) {
     	rightMargin = rectEnd - rectStart;
-    } else {
-	XtVaGetValues(window->textArea, textNcolumns, &nCols,
-    		textNwrapMargin, &wrapMargin, NULL);
-	rightMargin = (wrapMargin == 0 ? nCols : wrapMargin) - 1;
+    } else
+    {
+        XtVaGetValues(window->textArea,
+                textNcolumns, &nCols,
+                textNwrapMargin, &wrapMargin,
+                NULL);
+        rightMargin = (wrapMargin == 0 ? nCols : wrapMargin);
     }
     
     /* Fill the text */
@@ -449,7 +465,7 @@ static int nextTab(int pos, int tabDist)
     return (pos/tabDist)*tabDist + tabDist;
 }
 
-static int countLines(char *text)
+static int countLines(const char *text)
 {
     int count = 1;
     
@@ -678,7 +694,7 @@ static char *fillParagraph(char *text, int leftMargin, int firstLineIndent,
     return outText;
 }
 
-char *makeIndentString(int indent, int tabDist, int allowTabs, int *nChars)
+static char *makeIndentString(int indent, int tabDist, int allowTabs, int *nChars)
 {
     char *indentString, *outPtr;
     int i;
