@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: textDrag.c,v 1.4 2001/08/14 08:37:16 jlous Exp $";
+static const char CVSID[] = "$Id: textDrag.c,v 1.10 2004/07/21 11:32:05 yooden Exp $";
 /*******************************************************************************
 *									       *
 * textDrag.c - Text Dragging routines for NEdit text widget		       *
@@ -8,7 +8,8 @@ static const char CVSID[] = "$Id: textDrag.c,v 1.4 2001/08/14 08:37:16 jlous Exp
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
@@ -25,8 +26,18 @@ static const char CVSID[] = "$Id: textDrag.c,v 1.4 2001/08/14 08:37:16 jlous Exp
 * Written by Mark Edel							       *
 *									       *
 *******************************************************************************/
-#include <stdio.h>
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "textDrag.h"
+#include "textBuf.h"
+#include "textDisp.h"
+#include "textP.h"
+
 #include <limits.h>
+
 #include <X11/Intrinsic.h>
 #include <X11/IntrinsicP.h>
 #include <Xm/Xm.h>
@@ -34,10 +45,10 @@ static const char CVSID[] = "$Id: textDrag.c,v 1.4 2001/08/14 08:37:16 jlous Exp
 #if XmVersion >= 1002
 #include <Xm/PrimitiveP.h>
 #endif
-#include "textBuf.h"
-#include "textDisp.h"
-#include "textP.h"
-#include "textDrag.h"
+
+#ifdef HAVE_DEBUG_H
+#include "../debug.h"
+#endif
 
 static void trackModifyRange(int *rangeStart, int *modRangeEnd,
     	int *unmodRangeEnd, int modPos, int nInserted, int nDeleted);
