@@ -1,32 +1,42 @@
+/* $Id: textP.h,v 1.9 2004/07/21 11:32:05 yooden Exp $ */
 /*******************************************************************************
-*									       *
-* textP.h - Text Editing Widget						       *
-*									       *
-* Copyright (c) 1995 Universities Research Association, Inc.		       *
-* All rights reserved.							       *
-* 									       *
-* This material resulted from work developed under a Government Contract and   *
-* is subject to the following license:  The Government retains a paid-up,      *
-* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
-* works, perform publicly and display publicly by or for the Government,       *
-* including the right to distribute to other Government contractors.  Neither  *
-* the United States nor the United States Department of Energy, nor any of     *
-* their employees, makes any warrenty, express or implied, or assumes any      *
-* legal liability or responsibility for the accuracy, completeness, or         *
-* usefulness of any information, apparatus, product, or process disclosed, or  *
-* represents that its use would not infringe privately owned rights.           *
-*                                        				       *
-* Fermilab Nirvana GUI Library						       *
-* June 15, 1995								       *
-*									       *
-* Written by Mark Edel							       *
-*									       *
+*                                                                              *
+* textP.h -- Nirvana Editor Text Editing Widget private include file           *
+*                                                                              *
+* Copyright 2003 The NEdit Developers                                          *
+*                                                                              *
+* This is free software; you can redistribute it and/or modify it under the    *
+* terms of the GNU General Public License as published by the Free Software    *
+* Foundation; either version 2 of the License, or (at your option) any later   *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+*                                                                              *
+* This software is distributed in the hope that it will be useful, but WITHOUT *
+* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* for more details.                                                            *
+*                                                                              *
+* You should have received a copy of the GNU General Public License along with *
+* software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
+* Place, Suite 330, Boston, MA  02111-1307 USA                                 *
+*                                                                              *
+* Nirvana Text Editor                                                          *
+* July 31, 2001                                                                *
+*                                                                              *
 *******************************************************************************/
-#ifndef TEXTP_H
-#define TEXTP_H
 
-#include "text.h"
+#ifndef NEDIT_TEXTP_H_INCLUDED
+#define NEDIT_TEXTP_H_INCLUDED
+
+#include "textBuf.h"
+#include "textDisp.h"
+
+#include <X11/Intrinsic.h>
+#include <X11/Xlib.h>
+#include <X11/X.h>
 #include <Xm/XmP.h>
+#include <Xm/PrimitiveP.h>
+#include <X11/CoreP.h>
 
 enum dragStates {NOT_CLICKED, PRIMARY_CLICKED, SECONDARY_CLICKED,
 	CLICKED_IN_SELECTION,  PRIMARY_DRAG, PRIMARY_RECT_DRAG, SECONDARY_DRAG,
@@ -48,7 +58,7 @@ extern TextClassRec nTextClassRec;
 typedef struct _TextPart {
     /* resources */
     Pixel selectFGPixel, selectBGPixel, highlightFGPixel, highlightBGPixel;
-    Pixel cursorFGPixel;
+    Pixel cursorFGPixel, lineNumFGPixel, calltipFGPixel, calltipBGPixel;
     XFontStruct *fontStruct;
     Boolean pendingDelete;
     Boolean autoShowInsertPos;
@@ -60,12 +70,15 @@ typedef struct _TextPart {
     Boolean overstrike;
     Boolean heavyCursor;
     Boolean readOnly;
+    Boolean hidePointer;
     int rows, columns;
     int marginWidth, marginHeight;
     int cursorBlinkRate;
     int wrapMargin;
     int emulateTabs;
+    int lineNumCols;
     char *delimiters;
+    Cardinal cursorVPadding;
     Widget hScrollBar, vScrollBar;
     XtCallbackList focusInCB;
     XtCallbackList focusOutCB;
@@ -113,6 +126,7 @@ typedef struct _TextPart {
     	    	    	    	    	   source text was deleted */
     int dragSourceDeleted;  	    	/* # of chars. deleted "" */
     int dragNLines; 	    	    	/* # of newlines in text being drag'd */
+    XmString backlightCharTypes;	/* background class string to parse */
 } TextPart;
 
 typedef struct _TextRec {
@@ -121,4 +135,4 @@ typedef struct _TextRec {
    TextPart        text;
 } TextRec;
 
-#endif
+#endif /* NEDIT_TEXTP_H_INCLUDED */
