@@ -1,22 +1,65 @@
+static const char CVSID[] = "$Id: textSel.c,v 1.14 2004/07/21 11:32:05 yooden Exp $";
+/*******************************************************************************
+*									       *
+* textSel.c - Selection and clipboard routines for NEdit text widget		       *
+*									       *
+* Copyright (C) 1999 Mark Edel						       *
+*									       *
+* This is free software; you can redistribute it and/or modify it under the    *
+* terms of the GNU General Public License as published by the Free Software    *
+* Foundation; either version 2 of the License, or (at your option) any later   *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+* 									       *
+* This software is distributed in the hope that it will be useful, but WITHOUT *
+* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
+* for more details.							       *
+* 									       *
+* You should have received a copy of the GNU General Public License along with *
+* software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
+* Place, Suite 330, Boston, MA  02111-1307 USA		                       *
+*									       *
+* Nirvana Text Editor	    						       *
+* Dec. 15, 1995								       *
+*									       *
+* Written by Mark Edel							       *
+*									       *
+*******************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "textSel.h"
+#include "textP.h"
+#include "text.h"
+#include "textDisp.h"
+#include "textBuf.h"
+
 #include <stdio.h>
+#include <string.h>
 #include <limits.h>
+
 #include <Xm/Xm.h>
 #include <Xm/CutPaste.h>
+#include <Xm/Text.h>
 #include <X11/Xatom.h>
 #if XmVersion >= 1002
 #include <Xm/PrimitiveP.h>
 #endif
-#include "textBuf.h"
-#include "textDisp.h"
-#include "textP.h"
-#include "textSel.h"
+
+#ifdef HAVE_DEBUG_H
+#include "../debug.h"
+#endif
+
 
 #define N_SELECT_TARGETS 7
 #define N_CLIP_TARGETS 4
-#define N_ATOMS 10
+#define N_ATOMS 11
 enum atomIndex {A_TEXT, A_TARGETS, A_MULTIPLE, A_TIMESTAMP,
 	A_INSERT_SELECTION, A_DELETE, A_CLIPBOARD, A_INSERT_INFO,
-	A_ATOM_PAIR, A_MOTIF_DESTINATION};
+	A_ATOM_PAIR, A_MOTIF_DESTINATION, A_COMPOUND_TEXT};
 
 /* Results passed back to the convert proc processing an INSERT_SELECTION
    request, by getInsertSelection when the selection to insert has been
@@ -36,12 +79,13 @@ typedef struct {
     Time timeStamp;
     Widget widget;
     char *actionText;
+    int length;
 } selectNotifyInfo;
 
 static void modifiedCB(int pos, int nInserted, int nDeleted,
 	int nRestyled, char *deletedText, void *cbArg);
 static void sendSecondary(Widget w, Time time, Atom sel, int action,
-	char *actionText);
+	char *actionText, int actionTextLen);
 static void getSelectionCB(Widget w, XtPointer clientData, Atom *selType,
 	Atom *type, XtPointer value, unsigned long *length, int *format);
 static void getInsertSelectionCB(Widget w, XtPointer clientData,Atom *selType,
@@ -108,7 +152,7 @@ void CopyToClipboard(Widget w, Time time)
     char *text;
     long itemID = 0;
     XmString s;
-    int stat;
+    int stat, length;
     
     /* Get the selected text, if there's no selection, do nothing */
     text = BufGetSelectionText(((TextWidget)w)->text.textD->buffer);
@@ -117,20 +161,38 @@ void CopyToClipboard(Widget w, Time time)
     	return;
     }
 
+    /* If the string contained ascii-nul characters, something else was
+       substituted in the buffer.  Put the nulls back */
+    length = strlen(text);
+    BufUnsubstituteNullChars(text, ((TextWidget)w)->text.textD->buffer);
+    
+    /* Shut up LessTif */
+    if (XmClipboardLock(XtDisplay(w), XtWindow(w)) != ClipboardSuccess) {
+        XtFree(text);
+        return;
+    }
+    
     /* Use the XmClipboard routines to copy the text to the clipboard.
        If errors occur, just give up.  */
-    stat = XmClipboardStartCopy(XtDisplay(w), XtWindow(w),
-    	    s=XmStringCreateSimple("NEdit"), time, w, NULL, &itemID);
+    s = XmStringCreateSimple("NEdit");
+    stat = XmClipboardStartCopy(XtDisplay(w), XtWindow(w), s,
+    	    time, w, NULL, &itemID);
     XmStringFree(s);
     if (stat != ClipboardSuccess)
     	return;
+
+    /* Note that we were previously passing length + 1 here, but I suspect
+       that this was inconsistent with the somewhat ambiguous policy of
+       including a terminating null but not mentioning it in the length */
+
     if (XmClipboardCopy(XtDisplay(w), XtWindow(w), itemID, "STRING",
-    	    text, strlen(text)+1, 0, NULL) != ClipboardSuccess) {
+    	    text, length, 0, NULL) != ClipboardSuccess) {
     	XtFree(text);
     	return;
     }
     XtFree(text);
     XmClipboardEndCopy(XtDisplay(w), XtWindow(w), itemID);
+    XmClipboardUnlock(XtDisplay(w), XtWindow(w), False);
 }
 
 /*
@@ -160,7 +222,7 @@ void InsertPrimarySelection(Widget w, Time time, int isColumnar)
 void SendSecondarySelection(Widget w, Time time, int removeAfter)
 {
     sendSecondary(w, time, getAtom(XtDisplay(w), A_MOTIF_DESTINATION),
-    	    removeAfter ? REMOVE_SECONDARY : UNSELECT_SECONDARY, NULL);
+    	    removeAfter ? REMOVE_SECONDARY : UNSELECT_SECONDARY, NULL, 0);
 }
 
 /*
@@ -210,12 +272,11 @@ void MovePrimarySelection(Widget w, Time time, int isColumnar)
 ** Insert the X CLIPBOARD selection at the cursor position.  If isColumnar,
 ** do an BufInsertCol for a columnar paste instead of BufInsert.
 */
-void InsertClipboard(Widget w, Time time, int isColumnar)
+void InsertClipboard(Widget w, int isColumnar)
 {
     unsigned long length, retLength;
     textDisp *textD = ((TextWidget)w)->text.textD;
     textBuffer *buf = ((TextWidget)w)->text.textD->buffer;
-    selection *sel = &buf->primary;
     int cursorLineStart, column, cursorPos;
     char *string;
     long id = 0;
@@ -240,19 +301,33 @@ void InsertClipboard(Widget w, Time time, int isColumnar)
     	return;
     }
     string[retLength] = '\0';
-    
+
+    /* If the string contains ascii-nul characters, substitute something
+       else, or give up, warn, and refuse */
+    if (!BufSubstituteNullChars(string, retLength, buf)) {
+	fprintf(stderr, "Too much binary data, text not pasted\n");
+	XtFree(string);
+	return;
+    }
+
     /* Insert it in the text widget */
-    if (isColumnar) {
+    if (isColumnar && !buf->primary.selected) {
     	cursorPos = TextDGetInsertPosition(textD);
     	cursorLineStart = BufStartOfLine(buf, cursorPos);
     	column = BufCountDispChars(buf, cursorLineStart, cursorPos);
-    	BufInsertCol(buf, column, cursorLineStart, string, NULL, NULL);
+        if (((TextWidget)w)->text.overstrike) {
+	    BufOverlayRect(buf, cursorLineStart, column, -1, string, NULL,
+			   NULL);
+	} else {
+	    BufInsertCol(buf, column, cursorLineStart, string, NULL, NULL);
+	}
     	TextDSetInsertPosition(textD,
     	    	BufCountForwardDispChars(buf, cursorLineStart, column));
 	if (((TextWidget)w)->text.autoShowInsertPos)
     	    TextDMakeInsertPosVisible(textD);
     } else
-    	TextInsertAtCursor(w, string, NULL, True);
+    	TextInsertAtCursor(w, string, NULL, True,
+		((TextWidget)w)->text.autoWrapPastedText);
     XtFree(string);
 }
 
@@ -321,7 +396,7 @@ static void modifiedCB(int pos, int nInserted, int nDeleted,
 ** not NULL) when done.
 */
 static void sendSecondary(Widget w, Time time, Atom sel, int action,
-	char *actionText)
+	char *actionText, int actionTextLen)
 {
     static Atom selInfoProp[2] = {XA_SECONDARY, XA_STRING};
     Display *disp = XtDisplay(w);
@@ -355,6 +430,7 @@ static void sendSecondary(Widget w, Time time, Atom sel, int action,
     cbInfo->timeStamp = time;
     cbInfo->widget = (Widget)w;
     cbInfo->actionText = actionText;
+    cbInfo->length = actionTextLen;
     XtAddEventHandler(w, 0, True, selectNotifyEH, (XtPointer)cbInfo);
     cbInfo->timeoutProcID = XtAppAddTimeOut(context,
     	    XtAppGetSelectionTimeout(context),
@@ -385,9 +461,18 @@ static void getSelectionCB(Widget w, XtPointer clientData, Atom *selType,
        not be necessary, XLib documentation claims a NULL is already added,
        but the Xt documentation for this routine makes no such claim) */
     string = XtMalloc(*length + 1);
-    strncpy(string, (char *)value, *length);
+    memcpy(string, (char *)value, *length);
     string[*length] = '\0';
     
+    /* If the string contains ascii-nul characters, substitute something
+       else, or give up, warn, and refuse */
+    if (!BufSubstituteNullChars(string, *length, textD->buffer)) {
+	fprintf(stderr, "Too much binary data, giving up\n");
+	XtFree(string);
+	XtFree((char *)value);
+	return;
+    }
+    
     /* Insert it in the text widget */
     if (isColumnar) {
     	cursorPos = TextDGetInsertPosition(textD);
@@ -397,7 +482,8 @@ static void getSelectionCB(Widget w, XtPointer clientData, Atom *selType,
     	BufInsertCol(textD->buffer, column, cursorLineStart, string, NULL,NULL);
     	TextDSetInsertPosition(textD, textD->buffer->cursorPosHint);
     } else
-    	TextInsertAtCursor(w, string, NULL, False);
+    	TextInsertAtCursor(w, string, NULL, False,
+		((TextWidget)w)->text.autoWrapPastedText);
     XtFree(string);
     
     /* The selection requstor is required to free the memory passed
@@ -415,9 +501,7 @@ static void getSelectionCB(Widget w, XtPointer clientData, Atom *selType,
 static void getInsertSelectionCB(Widget w, XtPointer clientData,Atom *selType,
 	Atom *type, XtPointer value, unsigned long *length, int *format)
 {
-    textDisp *textD = ((TextWidget)w)->text.textD;
     textBuffer *buf = ((TextWidget)w)->text.textD->buffer;
-    selection *sel = &buf->primary;
     char *string;
     int *resultFlag = (int *)clientData;
  
@@ -431,11 +515,21 @@ static void getInsertSelectionCB(Widget w, XtPointer clientData,Atom *selType,
     
     /* Copy the string just to make space for the null character */
     string = XtMalloc(*length + 1);
-    strncpy(string, (char *)value, *length);
+    memcpy(string, (char *)value, *length);
     string[*length] = '\0';
     
+    /* If the string contains ascii-nul characters, substitute something
+       else, or give up, warn, and refuse */
+    if (!BufSubstituteNullChars(string, *length, buf)) {
+	fprintf(stderr, "Too much binary data, giving up\n");
+	XtFree(string);
+	XtFree((char *)value);
+	return;
+    }
+    
     /* Insert it in the text widget */
-    TextInsertAtCursor(w, string, NULL, True);
+    TextInsertAtCursor(w, string, NULL, True,
+	    ((TextWidget)w)->text.autoWrapPastedText);
     XtFree(string);
     *resultFlag = SUCCESSFUL_INSERT;
     
@@ -466,7 +560,7 @@ static void getExchSelCB(Widget w, XtPointer clientData, Atom *selType,
        this widget's secondary selection.  When complete, replace this
        widget's secondary selection with text "value" and free it. */
     sendSecondary(w, XtLastTimestampProcessed(XtDisplay(w)), XA_PRIMARY,
-    	    EXCHANGE_SECONDARY, (char *)value);
+    	    EXCHANGE_SECONDARY, (char *)value, *length);
 }
 
 /*
@@ -489,12 +583,17 @@ static Boolean convertSelectionCB(Widget w, Atom *selType, Atom *target,
     int getFmt, result = INSERT_WAITING;
     XEvent nextEvent;
     
-    /* target is text or string */
-    if (*target == XA_STRING || *target == getAtom(display, A_TEXT)) {
+    /* target is text, string, or compound text */
+    if (*target == XA_STRING || *target == getAtom(display, A_TEXT) ||
+        *target == getAtom(display, A_COMPOUND_TEXT)) {
+        /* We really don't directly support COMPOUND_TEXT, but recent
+           versions gnome-terminal incorrectly ask for it, even though
+           don't declare that we do.  Just reply in string format. */
     	*type = XA_STRING;
     	*value = (XtPointer)BufGetSelectionText(buf);
     	*length = strlen((char *)*value);
     	*format = 8;
+	BufUnsubstituteNullChars(*value, buf);
     	return True;
     }
     
@@ -562,8 +661,15 @@ static Boolean convertSelectionCB(Widget w, Atom *selType, Atom *target,
 
 static void loseSelectionCB(Widget w, Atom *selType)
 {
-    ((TextWidget)w)->text.selectionOwner = False;
-    BufUnselect(((TextWidget)w)->text.textD->buffer);
+    TextWidget tw = (TextWidget)w;
+    selection *sel = &tw->text.textD->buffer->primary;
+    char zeroWidth = sel->rectangular ? sel->zeroWidth : 0;
+    
+    /* For zero width rect. sel. we give up the selection but keep the 
+        zero width tag. */
+    tw->text.selectionOwner = False;
+    BufUnselect(tw->text.textD->buffer);
+    sel->zeroWidth = zeroWidth;
 }
 
 /*
@@ -586,6 +692,7 @@ static Boolean convertSecondaryCB(Widget w, Atom *selType, Atom *target,
     *value = (XtPointer)BufGetSecSelectText(buf);
     *length = strlen((char *)*value);
     *format = 8;
+    BufUnsubstituteNullChars(*value, buf);
     return True;
 }
 
@@ -682,6 +789,7 @@ static void selectNotifyEH(Widget w, XtPointer data, XEvent *event,
     XSelectionEvent *e = (XSelectionEvent *)event;
     selectNotifyInfo *cbInfo = (selectNotifyInfo *)data;
     int selStart, selEnd;
+    char *string;
 
     /* Check if this was the selection request for which this handler was
        set up, if not, do nothing */
@@ -711,17 +819,24 @@ static void selectNotifyEH(Widget w, XtPointer data, XEvent *event,
     if (cbInfo->action == REMOVE_SECONDARY) {
     	BufRemoveSecSelect(buf);
     } else if (cbInfo->action == EXCHANGE_SECONDARY) {
-    	selStart = buf->secondary.start;
-	BufReplaceSecSelect(buf, cbInfo->actionText);
-	if (buf->secondary.rectangular) {
-	    /*... it would be nice to re-select, but probably impossible */
-	    TextDSetInsertPosition(((TextWidget)w)->text.textD,
-	    	    buf->cursorPosHint);
-	} else {
-	    selEnd = selStart + strlen(cbInfo->actionText);
-	    BufSelect(buf, selStart, selEnd);
-	    TextDSetInsertPosition(((TextWidget)w)->text.textD, selEnd);
-	}
+	string = XtMalloc(cbInfo->length + 1);
+	memcpy(string, cbInfo->actionText, cbInfo->length);
+	string[cbInfo->length] = '\0';
+	selStart = buf->secondary.start;
+	if (BufSubstituteNullChars(string, cbInfo->length, buf)) {
+	    BufReplaceSecSelect(buf, string);
+	    if (buf->secondary.rectangular) {
+		/*... it would be nice to re-select, but probably impossible */
+		TextDSetInsertPosition(((TextWidget)w)->text.textD,
+	    		buf->cursorPosHint);
+	    } else {
+		selEnd = selStart + cbInfo->length;
+		BufSelect(buf, selStart, selEnd);
+		TextDSetInsertPosition(((TextWidget)w)->text.textD, selEnd);
+	    }
+	} else
+	    fprintf(stderr, "Too much binary data\n");
+	XtFree(string);
     }
     BufSecondaryUnselect(buf);
     XtDisownSelection(w, XA_SECONDARY, e->time);
@@ -758,7 +873,7 @@ static Atom getAtom(Display *display, int atomNum)
     static Atom atomList[N_ATOMS] = {0};
     static char *atomNames[N_ATOMS] = {"TEXT", "TARGETS", "MULTIPLE",
     	    "TIMESTAMP", "INSERT_SELECTION", "DELETE", "CLIPBOARD",
-    	    "INSERT_INFO", "ATOM_PAIR", "MOTIF_DESTINATION"};
+    	    "INSERT_INFO", "ATOM_PAIR", "MOTIF_DESTINATION", "COMPOUND_TEXT"};
     
     if (atomList[atomNum] == 0)
     	atomList[atomNum] = XInternAtom(display, atomNames[atomNum], False);
