@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: undo.c,v 1.6 2001/08/14 08:37:16 jlous Exp $";
+static const char CVSID[] = "$Id: undo.c,v 1.17 2004/07/21 11:32:06 yooden Exp $";
 /*******************************************************************************
 *									       *
 * undo.c -- Nirvana Editor undo command					       *
@@ -8,8 +8,9 @@ static const char CVSID[] = "$Id: undo.c,v 1.6 2001/08/14 08:37:16 jlous Exp $";
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
-* 									       *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+*                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
@@ -25,6 +26,22 @@ static const char CVSID[] = "$Id: undo.c,v 1.6 2001/08/14 08:37:16 jlous Exp $";
 * Written by Mark Edel							       *
 *									       *
 *******************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "undo.h"
+#include "textBuf.h"
+#include "text.h"
+#include "nedit.h"
+#include "search.h"
+#include "window.h"
+#include "file.h"
+#include "userCmds.h"
+#include "preferences.h"
+
+#include <string.h>
 #ifdef VMS
 #include "../util/VMSparam.h"
 #else
@@ -32,16 +49,14 @@ static const char CVSID[] = "$Id: undo.c,v 1.6 2001/08/14 08:37:16 jlous Exp $";
 #include <sys/param.h>
 #endif
 #endif /*VMS*/
+
 #include <Xm/Xm.h>
 #include <Xm/Text.h>
-#include "textBuf.h"
-#include "text.h"
-#include "nedit.h"
-#include "undo.h"
-#include "search.h"
-#include "window.h"
-#include "file.h"
-#include "userCmds.h"
+
+#ifdef HAVE_DEBUG_H
+#include "../debug.h"
+#endif
+
 
 #define FORWARD 1
 #define REVERSE 2
@@ -50,7 +65,7 @@ static void addUndoItem(WindowInfo *window, UndoInfo *undo);
 static void addRedoItem(WindowInfo *window, UndoInfo *redo);
 static void removeUndoItem(WindowInfo *window);
 static void removeRedoItem(WindowInfo *window);
-static void appendDeletedText(WindowInfo *window, char *deletedText,
+static void appendDeletedText(WindowInfo *window, const char *deletedText,
 	int deletedLen, int direction);
 static void trimUndoList(WindowInfo *window, int maxLength);
 static int determineUndoType(int nInserted, int nDeleted);
@@ -59,7 +74,7 @@ static void freeUndoRecord(UndoInfo *undo);
 void Undo(WindowInfo *window)
 {
     UndoInfo *undo = window->undo;
-	int restoredTextLength;
+    int restoredTextLength;
     
     /* return if nothing to undo */
     if (undo == NULL)
@@ -76,18 +91,25 @@ void Undo(WindowInfo *window)
     BufReplace(window->buffer, undo->startPos, undo->endPos,
     	    (undo->oldText != NULL ? undo->oldText : ""));
     
-	restoredTextLength = undo->oldText != NULL ? strlen(undo->oldText) : 0;
-    /* position the cursor in the focus pane after the changed text
-       to show the user where the undo was done */
-    TextSetCursorPos(window->lastFocus, undo->startPos + restoredTextLength);
+    restoredTextLength = undo->oldText != NULL ? strlen(undo->oldText) : 0;
+    if (!window->buffer->primary.selected || GetPrefUndoModifiesSelection()) {
+	/* position the cursor in the focus pane after the changed text
+	   to show the user where the undo was done */
+	TextSetCursorPos(window->lastFocus, undo->startPos + 
+	        restoredTextLength);
+    }
     
-	if (restoredTextLength > 0) {
-		BufSelect(window->buffer, undo->startPos, undo->startPos + restoredTextLength);
-	}
-	else {
-		BufUnselect(window->buffer);
-	}
-	 
+    if (GetPrefUndoModifiesSelection()) {
+        if (restoredTextLength > 0) {
+    	    BufSelect(window->buffer, undo->startPos, undo->startPos + 
+	            restoredTextLength);
+        }
+        else {
+    	    BufUnselect(window->buffer);
+        }
+    }
+    MakeSelectionVisible(window, window->lastFocus);
+
     /* restore the file's unmodified status if the file was unmodified
        when the change being undone was originally made.  Also, remove
        the backup file, since the text in the buffer is now identical to
@@ -104,7 +126,7 @@ void Undo(WindowInfo *window)
 void Redo(WindowInfo *window)
 {
     UndoInfo *redo = window->redo;
-	int restoredTextLength;
+    int restoredTextLength;
 
     /* return if nothing to redo */
     if (window->redo == NULL)
@@ -119,18 +141,25 @@ void Redo(WindowInfo *window)
     BufReplace(window->buffer, redo->startPos, redo->endPos,
     	    (redo->oldText != NULL ? redo->oldText : ""));
     
- 	restoredTextLength = redo->oldText != NULL ? strlen(redo->oldText) : 0;
-    /* position the cursor in the focus pane after the changed text
-       to show the user where the redo was done */
-    TextSetCursorPos(window->lastFocus, redo->startPos + restoredTextLength);
-    
- 	if (restoredTextLength > 0) {
-		BufSelect(window->buffer, redo->startPos, redo->startPos + restoredTextLength);
-	}
-	else {
-		BufUnselect(window->buffer);
-	}
+    restoredTextLength = redo->oldText != NULL ? strlen(redo->oldText) : 0;
+    if (!window->buffer->primary.selected || GetPrefUndoModifiesSelection()) {
+	/* position the cursor in the focus pane after the changed text
+	   to show the user where the undo was done */
+	TextSetCursorPos(window->lastFocus, redo->startPos + 
+	        restoredTextLength);
+    }
+    if (GetPrefUndoModifiesSelection()) {
 
+        if (restoredTextLength > 0) {
+    	    BufSelect(window->buffer, redo->startPos, redo->startPos + 
+	            restoredTextLength);
+        }
+        else {
+    	    BufUnselect(window->buffer);
+        }
+    }
+    MakeSelectionVisible(window, window->lastFocus);
+    
     /* restore the file's unmodified status if the file was unmodified
        when the change being redone was originally made */	    
     if (redo->restoresToSaved)
@@ -145,11 +174,12 @@ void Redo(WindowInfo *window)
 ** SaveUndoInformation stores away the changes made to the text buffer.  As a
 ** side effect, it also increments the autoSave operation and character counts
 ** since it needs to do the classification anyhow.
+**
+** Note: This routine must be kept efficient.  It is called for every 
+**       character typed.
 */
-/* Note: This routine must be kept efficient.  It is called for every character
-   typed. */
 void SaveUndoInformation(WindowInfo *window, int pos, int nInserted,
-	int nDeleted, char *deletedText)
+	int nDeleted, const char *deletedText)
 {
     int newType, oldType;
     UndoInfo *u, *undo = window->undo;
@@ -271,6 +301,23 @@ void ClearRedoList(WindowInfo *window)
     	removeRedoItem(window);
 }
 
+/* 
+** DisableUnmodified
+** 
+** Remove the ability of a window to become "Unmodified" through a sequence
+** of undos.  This can happen if the file the window is editing gets deleted,
+** for example.
+*/
+void DisableUnmodified(WindowInfo *window)
+{
+    UndoInfo *undo = window->undo;
+    
+    while (undo != NULL){
+        undo->restoresToSaved = False;
+        undo = undo->next;
+    }
+}
+
 /*
 ** Add an undo record (already allocated by the caller) to the window's undo
 ** list if the item pushes the undo operation or character counts past the
@@ -281,7 +328,7 @@ static void addUndoItem(WindowInfo *window, UndoInfo *undo)
     
     /* Make the undo menu item sensitive now that there's something to undo */
     if (window->undo == NULL) {
-    	XtSetSensitive(window->undoItem, True);
+    	SetSensitive(window, window->undoItem, True);
 	SetBGMenuUndoSensitivity(window, True);
     }
 
@@ -309,7 +356,7 @@ static void addRedoItem(WindowInfo *window, UndoInfo *redo)
 {
     /* Make the redo menu item sensitive now that there's something to redo */
     if (window->redo == NULL) {
-    	XtSetSensitive(window->redoItem, True);
+    	SetSensitive(window, window->redoItem, True);
 	SetBGMenuRedoSensitivity(window, True);
     }
     
@@ -338,7 +385,7 @@ static void removeUndoItem(WindowInfo *window)
     
     /* if there are no more undo records left, dim the Undo menu item */
     if (window->undo == NULL) {
-    	XtSetSensitive(window->undoItem, False);
+    	SetSensitive(window, window->undoItem, False);
 	SetBGMenuUndoSensitivity(window, False);
     }
 }
@@ -356,7 +403,7 @@ static void removeRedoItem(WindowInfo *window)
     
     /* if there are no more redo records left, dim the Redo menu item */
     if (window->redo == NULL) {
-    	XtSetSensitive(window->redoItem, False);
+    	SetSensitive(window, window->redoItem, False);
 	SetBGMenuRedoSensitivity(window, False);
     }
 }
@@ -367,7 +414,7 @@ static void removeRedoItem(WindowInfo *window)
 ** for continuing of a string of one character deletes or replaces, but will
 ** work with more than one character.
 */
-static void appendDeletedText(WindowInfo *window, char *deletedText,
+static void appendDeletedText(WindowInfo *window, const char *deletedText,
 	int deletedLen, int direction)
 {
     UndoInfo *undo = window->undo;
