@@ -1,50 +1,57 @@
-static const char CVSID[] = "$Id: getfiles.c,v 1.15 2001/08/14 08:37:16 jlous Exp $";
-/******************************************************************************
-*                                                                             *
-* Getfiles.c -- File Interface Routines                                       *
-*                                                                             *
-* Copyright (C) 1999 Mark Edel						       *
-*									       *
+static const char CVSID[] = "$Id: getfiles.c,v 1.30 2004/08/01 10:06:12 yooden Exp $";
+/*******************************************************************************
+*                                                                              *
+* Getfiles.c -- File Interface Routines                                        *
+*                                                                              *
+* Copyright (C) 1999 Mark Edel                                                 *
+*                                                                              *
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
-* 									       *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+*                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
-* for more details.							       *
-* 									       *
+* for more details.                                                            *
+*                                                                              *
 * You should have received a copy of the GNU General Public License along with *
 * software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
-* Place, Suite 330, Boston, MA  02111-1307 USA		                       *
-*									       *
-* Nirvana Text Editor	    						       *
-* May 23, 1991                                                                *
-*                                                                             *
-* Written by Donna Reid                                                       *
-*									      *
-* modified 11/5/91 by JMK: integrated changes made by M. Edel; updated for    *
-*			   destroy widget problem (took out ManageModalDialog *
-*			   call; added comments.			      *
-*	   10/1/92 by MWE: Added help dialog and fixed a few bugs             *
-*	    4/7/93 by DR:  Port to VMS				              *
-*	    6/1/93 by JMK: Integrate Port and changes by MWE to make          *
-*			   directories "sticky" and a fix to prevent opening  *
-*			   a directory when no filename was specified	      *
-*	   6/24/92 by MWE: Made filename list and directory list typeable,    *
-*			   set initial focus to filename list		      *
-*          6/25/93 by JMK: Fix memory leaks found by Purify.                  *
-*						                              *
-* Included are two routines written using Motif for accessing files:          *
-*									      *
-* GetExistingFilename  presents a FileSelectionBox dialog where users can     *
-*                      choose an existing file to open.                       *
-*									      *
-* GetNewFilename       presents a FileSelectionBox dialog to help the user    *
-*                      find a place for a new file.                           *
-*                                                                             *
-******************************************************************************/
+* Place, Suite 330, Boston, MA  02111-1307 USA                                 *
+*                                                                              *
+* Nirvana Text Editor                                                          *
+* May 23, 1991                                                                 *
+*                                                                              *
+* Written by Donna Reid                                                        *
+*                                                                              *
+* modified 11/5/91 by JMK: integrated changes made by M. Edel; updated for     *
+*                          destroy widget problem (took out ManageModalDialog  *
+*                          call; added comments.                               *
+*          10/1/92 by MWE: Added help dialog and fixed a few bugs              *
+*           4/7/93 by DR:  Port to VMS                                         *
+*           6/1/93 by JMK: Integrate Port and changes by MWE to make           *
+*                          directories "sticky" and a fix to prevent opening   *
+*                          a directory when no filename was specified          *
+*          6/24/92 by MWE: Made filename list and directory list typeable,     *
+*                          set initial focus to filename list                  *
+*          6/25/93 by JMK: Fix memory leaks found by Purify.                   *
+*                                                                              *
+* Included are two routines written using Motif for accessing files:           *
+*                                                                              *
+* GetExistingFilename  presents a FileSelectionBox dialog where users can      *
+*                      choose an existing file to open.                        *
+*                                                                              *
+*******************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "getfiles.h"
+#include "fileUtils.h"
+#include "misc.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -63,6 +70,7 @@ static const char CVSID[] = "$Id: getfiles.c,v 1.15 2001/08/14 08:37:16 jlous Ex
 #endif
 #endif /*VMS*/
 #include <sys/stat.h>
+
 #include <X11/keysym.h>
 #include <Xm/Xm.h>
 #include <Xm/PushBG.h>
@@ -71,9 +79,10 @@ static const char CVSID[] = "$Id: getfiles.c,v 1.15 2001/08/14 08:37:16 jlous Ex
 #include <Xm/Text.h>
 #include <Xm/MessageB.h>
 #include <Xm/List.h>
-#include "fileUtils.h"
-#include "misc.h"
-#include "getfiles.h"
+
+#ifdef HAVE_DEBUG_H
+#include "../debug.h"
+#endif
 
 #define MAX_ARGS 20			/* Maximum number of X arguments */
 #define PERMS 0666     			/* UNIX file permission, RW for owner,
@@ -265,38 +274,34 @@ int GetExistingFilename (Widget parent, char *promptString, char *filename)
     int       n;                      /* number of arguments               */
     Arg	      args[MAX_ARGS];	      /* arg list	                   */
     Widget    existFileSB;	      /* widget file select box 	   */
-    XmString  labelString;            /* compound string for prompt label  */
     XmString  titleString;	      /* compound string for dialog title  */
 
     n = 0;
-    labelString = XmStringCreateSimple(promptString);
-    titleString = XmStringCreateSimple(" ");
-    XtSetArg(args[n], XmNlistLabelString, labelString); n++;
+    titleString = XmStringCreateSimple(promptString);
     XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
     XtSetArg(args[n], XmNdialogTitle, titleString); n++;
-    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
     existFileSB = CreateFileSelectionDialog(parent,"FileSelect",args,n);
-    XmStringFree(labelString);
     XmStringFree(titleString);
 #ifndef SGI_CUSTOM
     if (RemoveRedundantTextField)
-    	XtUnmanageChild(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_TEXT)); 
-    XtUnmanageChild(XmFileSelectionBoxGetChild(existFileSB,
-    	    XmDIALOG_SELECTION_LABEL));
-    XtVaSetValues(XmFileSelectionBoxGetChild(existFileSB,
-    	    XmDIALOG_FILTER_LABEL), XmNmnemonic, 'l', XmNuserData,
-    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_FILTER_TEXT), NULL);
-    XtVaSetValues(XmFileSelectionBoxGetChild(existFileSB,
-    	    XmDIALOG_DIR_LIST_LABEL), XmNmnemonic, 'D', XmNuserData,
-    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_DIR_LIST), NULL);
+        XtUnmanageChild(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_TEXT)); 
+    XtUnmanageChild(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_SELECTION_LABEL));
+
+    XtVaSetValues(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_FILTER_LABEL),
+            XmNmnemonic, 'l',
+            XmNuserData, XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_FILTER_TEXT),
+            NULL);
+    XtVaSetValues(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_DIR_LIST_LABEL),
+            XmNmnemonic, 'D',
+            XmNuserData, XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_DIR_LIST),
+            NULL);
     XtVaSetValues(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_LIST_LABEL),
-    	    XmNmnemonic, promptString[strspn(promptString, "lD")], XmNuserData,
-    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_LIST), NULL);
+            XmNmnemonic, promptString[strspn(promptString, "lD")],
+            XmNuserData, XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_LIST),
+            NULL);
     AddDialogMnemonicHandler(existFileSB, FALSE);
-    RemapDeleteKey(XmFileSelectionBoxGetChild(existFileSB,
-                    XmDIALOG_FILTER_TEXT));
-    RemapDeleteKey(XmFileSelectionBoxGetChild(existFileSB,
-                    XmDIALOG_TEXT));
+    RemapDeleteKey(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_FILTER_TEXT));
+    RemapDeleteKey(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_TEXT));
 #endif
     return HandleCustomExistFileSB(existFileSB, filename);
 }
@@ -424,82 +429,20 @@ int HandleCustomExistFileSB(Widget existFileSB, char *filename)
 	XmStringFree(cFileString);
 	XtFree(fileString);
     }
-    XtDestroyWidget(existFileSB);
+    /* Destroy the dialog _shell_ iso. the dialog. Normally, this shouldn't
+       be necessary as the shell is destroyed automatically when the dialog
+       is. However, due to a bug in various Lesstif versions, the latter 
+       messes up the grab cascades and leaves new windows without grabs, such
+       that they appear to be frozen. */
+    XtDestroyWidget(XtParent(existFileSB));
     return SelectResult;
 }
 
 
-/*    	GetNewFilename					                   */
-/*									   */
-/*  This routine will popup a file selection box so that the user can      */
-/*  select a file that will be, at a later time in the application,        */
-/*  created and written to.  After the user selects a file, GetNewFilename */
-/*  checks whether the file already exists, and if it does, asks the user  */
-/*  if he/she wants to overwrite the file.  Answering no, allows the user  */
-/*  to select a new filename.  GetNewFilename also checks that the file    */
-/*  name specified by the user can be created, and allows re-entry if not. */
-/*  When the user presses the OK button to a filename satisying the above  */
-/*  criteria, GetNewFilename returns the selected filename and GFN_OK.     */
-/*  If the user presses the cancel button, the return value is GFN_CANCEL, */
-/*  and the filename character string supplied in the call is not altered. */
-/*  									   */
-/*  Arguments:								   */
-/*									   */
-/*	Widget  parent	      - parent widget id			   */
-/*	char *  promptString  - prompt string				   */
-/*	char *  filename      - a string to receive the selected filename  */
-/*				(this string will not be altered if the    */
-/*				user pressed the cancel button)		   */
-/*									   */
-/*  Returns:	GFN_OK	      - file was selected and OK button pressed	   */
-/*		GFN_CANCEL    - Cancel button pressed and no returned file */
-
-int GetNewFilename (Widget parent, char *promptString, char *filename)
-{
-    int       n;                      /* number of arguments               */ 
-    Arg       args[MAX_ARGS];	      /* arg list                          */
-    XmString  labelString;            /* compound string for prompt label  */
-    XmString  titleString;	      /* compound string for dialog title  */
-    Widget    newFileSB;              /* widget file select box for	   */
-    
-    n = 0;
-    labelString = XmStringCreateLtoR (promptString, 
-                  XmSTRING_DEFAULT_CHARSET);
-    titleString = XmStringCreateLtoR (" ", XmSTRING_DEFAULT_CHARSET);
-    XtSetArg(args[n], XmNselectionLabelString, labelString); n++;     
-    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
-    XtSetArg(args[n], XmNdialogTitle, titleString); n++;
-    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
-    newFileSB=CreateFileSelectionDialog(parent,"FileSelect",args,n);
-    XmStringFree(labelString);
-    XmStringFree(titleString);
-    XtVaSetValues(XmFileSelectionBoxGetChild(newFileSB,
-    	    XmDIALOG_FILTER_LABEL), XmNmnemonic, 'l', XmNuserData,
-    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_FILTER_TEXT), NULL);
-    XtVaSetValues(XmFileSelectionBoxGetChild(newFileSB,
-    	    XmDIALOG_DIR_LIST_LABEL), XmNmnemonic, 'D', XmNuserData,
-    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_DIR_LIST), NULL);
-    XtVaSetValues(XmFileSelectionBoxGetChild(newFileSB,
-    	    XmDIALOG_LIST_LABEL), XmNmnemonic, 'F', XmNuserData,
-    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_LIST), NULL);
-    XtVaSetValues(XmFileSelectionBoxGetChild(newFileSB,
-    	    XmDIALOG_SELECTION_LABEL), XmNmnemonic,
-    	    promptString[strspn(promptString, "lFD")], XmNuserData,
-    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_TEXT), NULL);
-    AddDialogMnemonicHandler(newFileSB, FALSE);
-    RemapDeleteKey(XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_FILTER_TEXT));
-    RemapDeleteKey(XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_TEXT));
-    return HandleCustomNewFileSB(newFileSB, filename, NULL);
-}
-
 /*
 ** HandleCustomNewFileSB
 **
 ** Manage a customized file selection box for opening new files.
-** Use this if you want to change the standard file selection dialog
-** from the defaults provided in GetNewFilename, but still
-** want take advantage of the button processing, help messages, and
-** file checking of GetExistingFilename.
 **
 **  Arguments:
 **
@@ -601,7 +544,13 @@ int HandleCustomNewFileSB(Widget newFileSB, char *filename, char *defaultName)
 	DefaultPattern = cPattern;
 	XmStringGetLtoR(cFileString, XmSTRING_DEFAULT_CHARSET, &fileString);
 	/* See note in existing file routines about Motif 2.x bug. */
+#ifdef VMS
+	/* VMS  won't return `/' as the 1st character of the full file spec.
+	 `:' terminates the device name and is not allowed elsewhere */
+	if (strchr(fileString, ':') != NULL) {
+#else
 	if (fileString[0] == '/') {
+#endif /* VMS */
 	    /* The directory name is already present in the file name or
 	       the user entered a full path name. */
 	    strcpy(filename, fileString);
@@ -620,9 +569,9 @@ int HandleCustomNewFileSB(Widget newFileSB, char *filename, char *defaultName)
 }
 
 /*
-** Return current default directory used by GetExistingFilename and
-** GetNewFilename.  Can return NULL if no default directory has been set
-** (meaning use the application's current working directory) String must
+** Return current default directory used by GetExistingFilename.
+** Can return NULL if no default directory has been set (meaning
+** use the application's current working directory) String must
 ** be freed by the caller using XtFree.
 */
 char *GetFileDialogDefaultDirectory(void)
@@ -636,10 +585,10 @@ char *GetFileDialogDefaultDirectory(void)
 }
 
 /*
-** Return current default match pattern used by GetExistingFilename and
-** GetNewFilename.  Can return NULL if no default pattern has been set
-** (meaning use a pattern matching all files in the directory) String must
-** be freed by the caller using XtFree.
+** Return current default match pattern used by GetExistingFilename.
+** Can return NULL if no default pattern has been set (meaning use
+** a pattern matching all files in the directory) String must be
+** freed by the caller using XtFree.
 */
 char *GetFileDialogDefaultPattern(void)
 {
@@ -652,9 +601,9 @@ char *GetFileDialogDefaultPattern(void)
 }
 
 /*
-** Set the current default directory to be used by GetExistingFilename and
-** GetNewFilename.  "dir" can be passed as NULL to clear the current default
-** directory and use the application's working directory instead.
+** Set the current default directory to be used by GetExistingFilename.
+** "dir" can be passed as NULL to clear the current default directory
+** and use the application's working directory instead.
 */
 void SetFileDialogDefaultDirectory(char *dir)
 {
@@ -664,9 +613,9 @@ void SetFileDialogDefaultDirectory(char *dir)
 }
 
 /*
-** Set the current default match pattern to be used by GetExistingFilename and
-** GetNewFilename.  "pattern" can be passed as NULL as the equivalent a pattern
-** matching all files in the directory.
+** Set the current default match pattern to be used by GetExistingFilename.
+** "pattern" can be passed as NULL as the equivalent a pattern matching
+** all files in the directory.
 */
 void SetFileDialogDefaultPattern(char *pattern)
 {
@@ -734,8 +683,11 @@ static void createErrorDialog(Widget parent)
     	    NULL);
     XtUnmanageChild(XmMessageBoxGetChild(ErrorDialog, XmDIALOG_OK_BUTTON));
     XtUnmanageChild(XmMessageBoxGetChild(ErrorDialog, XmDIALOG_HELP_BUTTON));
-    buttonString = XmStringCreateLtoR ("Dismiss", XmSTRING_DEFAULT_CHARSET);
+    buttonString = XmStringCreateLtoR("OK", XmSTRING_DEFAULT_CHARSET);
     XtVaSetValues(ErrorDialog, XmNcancelLabelString, buttonString, NULL);
+    XtVaSetValues(XmMessageBoxGetChild(ErrorDialog, XmDIALOG_CANCEL_BUTTON),
+            XmNmarginWidth, BUTTON_WIDTH_MARGIN,
+            NULL);
     XmStringFree(buttonString);
 }
 
@@ -926,9 +878,10 @@ static Widget createPanelHelp(Widget parent, const char *helpText, const char *t
     ac = 0;
     XtSetArg (al[ac], XmNbottomAttachment, XmATTACH_FORM);  ac++;
     XtSetArg (al[ac], XmNtopAttachment, XmATTACH_NONE);  ac++;
-    XtSetArg(al[ac], XmNlabelString, st1=XmStringCreateLtoR ("Dismiss", 
+    XtSetArg(al[ac], XmNlabelString, st1=XmStringCreateLtoR ("OK", 
                       XmSTRING_DEFAULT_CHARSET)); ac++;
-    button = XmCreatePushButtonGadget(form, "dismiss", al, ac);
+    XtSetArg (al[ac], XmNmarginWidth, BUTTON_WIDTH_MARGIN);  ac++;
+    button = XmCreatePushButtonGadget(form, "ok", al, ac);
     XtAddCallback(button, XmNactivateCallback, (XtCallbackProc)helpDismissCB,
     	    (char *)form);
     XmStringFree(st1);
@@ -951,6 +904,7 @@ static Widget createPanelHelp(Widget parent, const char *helpText, const char *t
     XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM);  ac++;
     XtSetArg(al[ac], XmNbottomWidget, button);  ac++;
     text = XmCreateScrolledText(form, "helpText", al, ac);
+    AddMouseWheelSupport(text);
     XtManageChild(text);
     
     SET_ONE_RSRC(XtParent(form), XmNtitle, title);
@@ -975,6 +929,7 @@ static void makeListTypeable(Widget listW)
 ** Action procedure for processing characters typed in a list, finds the
 ** first item matching the characters typed so far.
 */
+static int nKeystrokes = 0; /* Global key stroke history counter */
 static void listCharEH(Widget w, XtPointer callData, XEvent *event,
 	Boolean *continueDispatch)
 {
@@ -984,7 +939,6 @@ static void listCharEH(Widget w, XtPointer callData, XEvent *event,
     KeySym kSym;
     char name[MAXPATHLEN], path[MAXPATHLEN];
     static char keystrokes[MAX_LIST_KEYSTROKES];
-    static int nKeystrokes = 0;
     static Time lastKeyTime = 0;
     
     /* Get the ascii character code represented by the event */
@@ -1030,7 +984,10 @@ static void listCharEH(Widget w, XtPointer callData, XEvent *event,
     selectPos = 0;
     for (i=0; i<nItems; i++) {
     	XmStringGetLtoR(items[i], XmSTRING_DEFAULT_CHARSET, &itemString);
-    	ParseFilename(itemString, name, path);
+    	if (ParseFilename(itemString, name, path) != 0) {
+	   XtFree(itemString);
+	   return;
+	}
 	XtFree(itemString);
     	cmp = strncmp(name, keystrokes, nKeystrokes);
     	if (cmp == 0) {
@@ -1077,6 +1034,8 @@ static void replacementDirSearchProc(Widget w, XtPointer searchData)
     
     /* Call the original search procedure to do the actual search */
     (*OrigDirSearchProc)(w, searchData);
+    /* Refreshing a list clears the keystroke history, even if no update. */
+    nKeystrokes = 0;
     XtVaGetValues(w, XmNlistUpdated, &updated, NULL);
     if (!updated)
     	return;
@@ -1091,6 +1050,8 @@ static void replacementFileSearchProc(Widget w, XtPointer searchData)
     
     /* Call the original search procedure to do the actual search */
     (*OrigFileSearchProc)(w, searchData);
+    /* Refreshing a list clears the keystroke history, even if no update. */
+    nKeystrokes = 0;
     XtVaGetValues(w, XmNlistUpdated, &updated, NULL);
     if (!updated)
     	return;
